".First.lib"<-
function(library, section)
{
  if(version$major == 3) {
    if(version$minor < 2)
      stop("This library needs S-Plus 3.2 or later")
    else dyn.load(paste(library, section, "sp3\\survcart.obj", sep = "\\"))
  }
  if(version$major == 4)
    dyn.load(paste(library, section, "sp4\\survcart.obj", sep = "\\"))
}

"[.survtree"<-
function(x, ..., drop = T)
{
  #may want to look at [.data.frame to translate logicals to numerics
  nodes <- unlist(...)
  if(is.null(nodes)) stop("Nodes must be supplied")
  if(all(nodes < 0))
    TT <- substitute(snip.survtree(survtree = x, nodes =  - nodes))
  else if(all(nodes > 0))
    TT <- substitute(select.tree(tree = x, nodes = nodes, drop = drop))
  else stop("Nodes must all have the same sign")
  eval(TT, sys.parent())
}

"browser.survtree"<-
function(survtree, nodes, digits = .Options$digits - 3)
{
  survtree.test(survtree)
  frame <- survtree$frame
  node <- row.names(frame)
  if(!missing(nodes)) return(frame[node.match(nodes, node),  ])
  labs <- c("node number:", "split:", "n:", "dev:", "log.rank.value:",
	    "log.rank.alpha:")
  is.prob <- !is.null(ylevels <- attr(survtree, "ylevels"))
  if(is.prob) frame$yval <- as.character(frame$yval)
  else frame$yval <- format(signif(frame$yval, digits = digits))
  frame$dev <- format(signif(frame$dev, digits = digits))
  xy <- treeco(survtree)
  splits <- labels(survtree, pretty = 0)
  while(length(i <- identify(xy, n = 1, plot = F)) > 0) {
    fi <- frame[i,  , drop = F]
    info <- c(node[i], splits[i], fi$n, fi$dev, fi$log.rank.value, fi$log.rank.alpha)
    cat(paste(labs, info, "\n"))
    if(is.prob) print(fi$yprob)
  }
  invisible()
}

"data.survtree"<-
function(survtree)
{
  oc <- survtree$call
  while(oc[[1]] != "survtree") oc <- eval(oc[[2]])$call
  if(is.null(oc$data))
    NULL
  else {
    warning(paste("Retrieving data from", oc$data))
    eval(oc$data)
  }
}

"descendants"<-
function(nodes, include = T)
{
  nodes <- as.numeric(nodes)	
  #returns a square logical matrix, one row for each node, giving the postions of descendent nodes
  #the colums contain the ancestors of each node
  n <- length(nodes)
  ind <- 1:n
  desc <- matrix(F, n, n)
  if(include) diag(desc) <- T
  parents <- match((nodes %/% 2), nodes)
  lev <- floor(log(nodes, base = 2))
  desc[1, 2:n] <- T
  for(i in max(lev):2) {
    desc[cbind(ind[parents[lev == i]], ind[lev == i])] <- T
    parents[lev == i] <- parents[parents[lev == i]]
    lev[lev == i] <- i - 1
  }
  return(desc)
}

"edit.survtree"<-
function(object, node, var, splitl, splitr = NULL)
{
  survtree.test(object)
  if(inherits(node, "tree.split")) {
    var <- names(node)[1]
    if(is.null(splitl <- node$cutleft)) splitl <- node[, var]
    else splitr <- node$cutright
    node <- attr(node, "node")
  }
  call <- object$call
  x <- model.frame(object)
  if(is.null(y <- object$y)) y <- model.extract(x, "response")
  x <- x[var]
  levx <- levels(x[, 1])
  if(node == 1) obs <- names(object$where)
  else obs <- identify.survtree(object, node)
  if(!is.null(levx)) {
    lets <- letters[match(as.vector(x[obs,  ]), levx)]
    nl <- nchar(splitl)
    subl <- obs[grep(substring(splitl, 1:nl, 1:nl), lets)]
    nr <- nchar(splitr)
    subr <- obs[grep(substring(splitr, 1:nr, 1:nr), lets)]
  } else {
    subl <- obs[x[obs,  ] < splitl]
    subr <- obs[x[obs,  ] >= splitl]
  }
  call$subset <- subl
  subtree <- eval(call)
  object <- graft.survtree(object, subtree, where = 2 * node)
  call$subset <- subr
  subtree <- eval(call)
  object <- graft.survtree(object, subtree, where = (2 * node) + 1)
  rn <- match(node, row.names(object$frame))
  object$frame$var[rn] <- var
  if(is.null(levx)) split <- paste(c("<", ">"), splitl, sep = "")
  else split <- paste(":", c(splitl, splitr), sep = "")
  object$frame$splits[rn,  ] <- split
  object$call <- match.call()
  object
}

"graft.survtree"<-
function(survtree, subtree, where)
{
  rbind.data.frame <- function(..., deparse.level = 1)
  {
    match.names <- function(clabs, nmi)
    {
      if(all(clabs == nmi)) NULL
      else if(all(nii <- match(nmi, clabs, 0))) nii
      else stop(paste("names don't match previous names:\n\t", 
                      paste(nmi[nii == 0], collapse = ", ")))
    }
    Make.row.names <- function(nmi, ri, ni, nrow)
    {
      if(nchar(nmi)) {
        if(ni > 1)
          paste(nmi, ri, sep = ".")
        else nmi
      }
      else if(nrow > 0 && all(ri == seq(length = ni)))
        seq(from = nrow + 1, length = ni)
      else ri
    }
    n <- nargs()
    if(!n)
      return(structure(list(), class = "data.frame", 
                       row.names = character()))
    all <- list(...)
    nms <- names(all)
    if(is.null(nms)) nms <- character(length(all))
    cl <- NULL
    perm <- rows <- rlabs <- vector("list", n)
    nrow <- 0
    value <- clabs <- NULL
    all.levs <- list()
    for(i in 1:n) {
      #check the arguments, develop row and column labels
      xi <- all[[i]]
      nmi <- nms[i]
      if(inherits(xi, "data.frame")) {
        if(is.null(cl))
          cl <- class(xi)
        ri <- row.names(xi)
        ni <- length(ri)
        if(is.null(clabs))
          clabs <- names(xi)
        else perm[[i]] <- pi <- match.names(clabs, names(xi))
        rows[[i]] <- nii <- seq(from = nrow + 1, length = ni)
        rlabs[[i]] <- Make.row.names(nmi, ri, ni, nrow)
        nrow <- nrow + ni
        if(is.null(value)) {
          value <- unclass(xi)
          nvar <- length(value)
          all.levs <- vector("list", nvar)
          has.dim <- logical(nvar)
          for(j in 1:nvar) {
            xj <- value[[j]]
            all.levs[[j]] <- levels(xj)
            has.dim[j] <- length(dim(xj)) == 2
          }
        }
        else for(j in 1:nvar)
          if(length(lij <- levels(xi[[j]]))) {
            if(is.null(pi) || is.na(jj <- pi[[j]]))
              jj <- j
            all.levs[[jj]] <- unique(c(all.levs[[jj]], lij))
          }
      }
      else if(is.list(xi)) {
        ni <- range(sapply(xi, length))
        if(ni[1] == ni[2]) ni <- ni[1]
        else stop("invalid list argument: all variables should have the same length")
        rows[[i]] <- ri <- seq(from = nrow + 1, length = ni)
        nrow <- nrow + ni
        rlabs[[i]] <- Make.row.names(nmi, ri, ni, nrow)
        if(length(nmi <- names(xi))) {
          if(is.null(clabs)) clabs <- nmi
          else perm[[i]] <- match.names(clabs, nmi)
        }
      }
      else if(length(xi)) {
        nrow <- nrow + 1
        rlabs[[i]] <- if(nchar(nmi)) nmi else nrow
      }
    }
    nvar <- length(clabs)
    if(nvar == 0)
      nvar <- max(sapply(all, length))  # only vector args
    if(nvar == 0)
      return(structure(list(), class = "data.frame", row.names = character()))
    pseq <- 1:nvar
    if(is.null(value)) {
      value <- list()
      value[pseq] <- list(logical(nrow))
    }
    names(value) <- clabs
    for(j in 1:nvar)
      if(length(lij <- all.levs[[j]])) value[[j]] <- 
        factor(as.vector(value[[j]]), lij, exclude = NULL)
    if(any(has.dim)) {
      rmax <- max(unlist(rows))
      for(i in (1:nvar)[has.dim])
        if(!inherits(xi <- value[[i]], "data.frame")) {
          dn <- dimnames(xi)
          row.names <- dn[[1]]
          if(length(row.names))
            length(row.names) <- rmax
          pi <- dim(xi)[2]
          length(xi) <- rmax * pi
          value[[i]] <- array(xi, c(rmax, pi), list(row.names, dn[[2]]))
        }
    }
    for(i in 1:n) {
      xi <- unclass(all[[i]])
      if(!is.list(xi))
        if(length(xi) != nvar)
          xi <- rep(xi, length = nvar)
      ri <- rows[[i]]
      pi <- perm[[i]]
      if(is.null(pi))
        pi <- pseq
      for(j in 1:nvar) {
        jj <- pi[j]
        if(has.dim[jj])
          value[[jj]][ri,  ] <- xi[[j]]
        else value[[jj]][ri] <- xi[[j]]
      }
    }
    for(j in 1:nvar) {
      xj <- value[[j]]
      if(!has.dim[j] && (is.character(xj) || is.logical(xj)))
        value[[j]] <- factor(xj)
    }
    rlabs <- unlist(rlabs)
    while(any(xj <- duplicated(rlabs)))
      rlabs[xj] <- paste(rlabs[xj], seq(length = sum(xj)), sep = "")
    if(is.null(cl)) {
      as.data.frame(value, row.names = rlabs)
    }
    else {
      class(value) <- cl	
      # ensure that row names are ok.  Similar to row.names<-	
      rlabs <- as.character(rlabs)
      if(any(duplicated(rlabs)))
        rlabs <- make.names(rlabs, uniq = T)
      attr(value, "row.names") <- rlabs
      value
    }
  }
  survtree.test(survtree)
  if(!inherits(subtree, "survtree") & !inherits(subtree, "singlenode"))
    stop("Not legitimate survtree")
  f <- survtree$frame
  if(f$var[match(where, row.names(f))] != "<leaf>") {
    #only need to snip if not at a terminal node
    survtree <- snip.survtree(survtree, nodes = where)
    f <- survtree$frame
  }
  nodes <- row.names(f)
  n <- match(where, nodes)
  subf <- subtree$frame
  snodes <- as.numeric(row.names(subf))
  sdepth <- tree.depth(snodes)
  pos <- snodes %% (2^sdepth)
  row.names(subf) <- pos + (where * (2^sdepth))
  survtree$frame <- rbind.data.frame(f[1:(n - 1),  ], subf)
  if(n != length(nodes)) {
    survtree$frame <- rbind.data.frame(survtree$frame,
                                       f[(n + 1):length(nodes),  ])
  }
  ind <- survtree$where > n
  survtree$where[ind] <- survtree$where[ind] + dim(subf)[1] - 1
  ind <- match(names(subtree$where), names(survtree$where))
  survtree$where[ind] <- subtree$where + n - 1
  survtree$call <- match.call()
  survtree
}

"graph.survtree"<-
function(tree, time, zens, limit, xtile, interactive = T, text.flag = T, 
	node.number.flag = T, node.total.no.flag = T, rootbox.size = 0.3, 
	box.height = 0.2, col.normal = 1, col.bad.right = 2, col.bad.left
	 = 3, col.boxconnection = 4, lty.boxconnection = 2)
{
  # +++++++++++++++++
  # +++ Strings   +++
  # +++++++++++++++++
  YOUR.CHOICE <- "Your choice: "
  ONLY.0.1 <- "Only 0 or 1 are allowed !\n"
  AGAIN <- "Try again.\n"
  ONLY.NUMBERS <- "Only numbers are allowed !\n"
  ONLY.GT.0 <- "Only values greater than 0 are allowed !\n"
  DEFAULT.VALUE <- "Default = "
  TAKE.DEFAULT.AT.MINUS.1 <- ", type -1 to take default !\n"
  ONLY.0.TO.100 <- "Only values from 0 to 100 are allowed !\n"
  ONLY.0.TO.1 <- "Only values from 0 to 1 are allowed !\n"
  O.K.TAKE.DEFAULT <- "O.K., I take default !\n"
  NO.ARGUMENTS.STOP <- 
    "\nNeither argument limit nor argument xtile given !\n"
  GIVE.LIMIT <- "\nNow type the timevalue: \n"
  GIVE.XTILE <- "\nNow type the percentile: \n"
  STRING.LIMIT.GIVEN <- "survival-rate at time "
  STRING.XTILE.GIVEN <- "time with survival-rate "
  CHOOSE.DRAW.SPLIT.CRITERIUM <- 
    "\nNow choose if the split-criterium will be added to the plot\n\n1 = yes, 0 = no \n"
  CHOOSE.GRAPH.PARAM <- "\nNow choose some graphical parameters !\n"
  CHOOSE.ROOTBOX.SIZE <- 
    "\nThe value : \n\"Width of the rootbox divided by width of the whole plot\"\n"
  CHOOSE.HEIGHT.BOX <- 
    "\nThe value : \n\"Height of a box divided by the distance between two splitting levels\"\n"
  DRAW.AGAIN <- "\nYou can now choose if you will plot again !\n0 = no, 1 = yes \n"
  # +++++++++++++++++
  # +++ Functions +++
  # +++++++++++++++++
  SWORK <- getenv("S_WORK")
  double.point <- grep(";", substring(SWORK, 1:nchar(SWORK), 1:nchar(SWORK)))
  if(length(double.point) > 0) {
    data.dir <- substring(SWORK, 1, double.point - 1)
  } else {
    data.dir <- SWORK
  }
  tmp.dir <- paste(data.dir, "CART-SD.TMP", sep = "\\")
  dos(paste("mkdir", tmp.dir))
  attach(tmp.dir, 1)	
  #	warning(paste("graph.survtree attached directory", tmp.dir, ".\nIf an error occures, use detach(1) to detach it from the search list!"))
  choice.info <<- function()
    {
      cat("\nPlease make your choice about the type of plot: \n")
      cat("0 : You will be asked for a percentile XTILE, \n")
      cat("    the meaning of the x-axis will then be: \n")
      cat("   \"time with survival-rate XTILE-percent\". \n")
      cat("1 : You will be asked for a timevalue LIMIT, \n")
      cat("    the meaning of the x-axis will then be: \n")
      cat("   \"survival-rate at time LIMIT\".\n")
      cat("\ntype 0 or 1 : \n")
    }
  ask.wahl <<- function(YOUR.CHOICE = substitute(YOUR.CHOICE, 
                          sys.nframe() - 1), ONLY.0.1 = substitute(ONLY.0.1, sys.nframe() - 1),
			AGAIN = substitute(AGAIN, sys.nframe() - 1),
			ONLY.NUMBERS = substitute(ONLY.NUMBERS, sys.nframe() - 1))
    {
      nochmal <- T
      while(nochmal == T) {
	cat(YOUR.CHOICE)
	ans <- eval(parse())
	if(mode(ans) == "numeric")
	  if(ans[1] == 0 || ans[1] == 1)
	    nochmal <- F
          else {
            cat(ONLY.0.1)
            cat(AGAIN)
          }
        else {
          cat(ONLY.NUMBERS)
          cat(AGAIN)
        }
      }
      ans
    }
  ask.limit <<-
    function(
             YOUR.CHOICE = substitute(YOUR.CHOICE, sys.nframe() - 1),
             ONLY.GT.0 = substitute(ONLY.GT.0, sys.nframe() - 1),
             AGAIN = substitute(AGAIN, sys.nframe() - 1),
             ONLY.NUMBERS = substitute(ONLY.NUMBERS, sys.nframe() - 1)
             )
  {
    nochmal <- T
    while(nochmal == T) {
      cat(YOUR.CHOICE)
      ans <- eval(parse())
      if(mode(ans) == "numeric")
        if(ans[1] > 0) nochmal <- F
        else {
          cat(ONLY.GT.0)
          cat(AGAIN)
        }
      else {
        cat(ONLY.NUMBERS)
        cat(AGAIN)
      }
    }
    ans
  }
  ask.xtile <<-
    function(std,
             DEFAULT.VALUE = substitute(DEFAULT.VALUE, sys.nframe() - 1),
             TAKE.DEFAULT.AT.MINUS.1 = substitute(TAKE.DEFAULT.AT.MINUS.1, sys.nframe() - 1),
             YOUR.CHOICE = substitute(YOUR.CHOICE, sys.nframe() - 1),
             ONLY.0.TO.100 = substitute(ONLY.0.TO.100, sys.nframe() - 1),
             AGAIN = substitute(AGAIN, sys.nframe() - 1),
             ONLY.NUMBERS = substitute(ONLY.NUMBERS, sys.nframe() - 1),
             O.K.TAKE.DEFAULT = substitute(O.K.TAKE.DEFAULT, sys.nframe() - 1)
             )
  {
    cat(DEFAULT.VALUE, std, TAKE.DEFAULT.AT.MINUS.1)
    nochmal <- T
    while(nochmal == T) {
      cat(YOUR.CHOICE)
      ans <- eval(parse())
      if(mode(ans) == "numeric")
        if(ans[1] != -1)
          if(ans[1] >= 0 && ans[1] <= 100) {
            nochmal <- F
          }
          else {
            cat(ONLY.0.TO.100)
            cat(AGAIN)
          }
        else {
          nochmal <- F
          cat(O.K.TAKE.DEFAULT)
          ans <- std
        }
      else {
        cat(ONLY.NUMBERS)
        cat(AGAIN)
      }
    }
    ans
  }
  ask.funct <<- function(std, DEFAULT.VALUE = substitute(
				DEFAULT.VALUE, sys.nframe() - 1), TAKE.DEFAULT.AT.MINUS.1
			 = substitute(TAKE.DEFAULT.AT.MINUS.1, sys.nframe() - 1), 
			 YOUR.CHOICE = substitute(YOUR.CHOICE, sys.nframe() - 1), 
			 ONLY.0.TO.1 = substitute(ONLY.0.TO.1, sys.nframe() - 1), 
			 AGAIN = substitute(AGAIN, sys.nframe() - 1), ONLY.NUMBERS
			 = substitute(ONLY.NUMBERS, sys.nframe() - 1), 
			 O.K.TAKE.DEFAULT = substitute(O.K.TAKE.DEFAULT, sys.nframe(
			   ) - 1))
    {
      cat(DEFAULT.VALUE, std, TAKE.DEFAULT.AT.MINUS.1)
      nochmal <- T
      while(nochmal == T) {
	cat(YOUR.CHOICE)
	ans <- eval(parse())
	if(mode(ans) == "numeric")
	  if(ans[1] != -1)
	    if(ans[1] >= 0 && ans[1] <= 1) {
	      nochmal <- F
	    }
            else {
              cat(ONLY.0.TO.1)
              cat(AGAIN)
            }
          else {
            nochmal <- F
            cat(O.K.TAKE.DEFAULT)
            ans <- std
          }
        else {
          cat(ONLY.NUMBERS)
          cat(AGAIN)
        }
      }
      ans
    }
  get.x.time.surv.fit <<- function(elem.node.surv.fit, xtile, 
				   col.bad.right, col.bad.left, col.normal)
    {
      col.val <- col.normal
      val <- max(elem.node.surv.fit$time[elem.node.surv.fit$surv == 
					 xtile])
      if(is.na(val)) {
	val.high <- max(elem.node.surv.fit$time[
						elem.node.surv.fit$surv > xtile])
	if(is.na(val.high)) {
	  val.high <- elem.node.surv.fit$time[1]
	  col.val <- col.bad.left
	}
	val.low <- min(elem.node.surv.fit$time[
					       elem.node.surv.fit$surv < xtile])
	if(is.na(val.low)) {
	  val.low <- elem.node.surv.fit$time[length(
						    elem.node.surv.fit$time)]
	  col.val <- col.bad.right
	}
	val <- 0.5 * (val.high + val.low)
      }
      attributes(val)$col <- col.val
      val
    }
  get.x.rate.surv.fit <<- function(elem.node.surv.fit, limit, 
				   col.bad.right, col.bad.left, col.normal)
    {
      col.val <- col.normal
      val <- max(elem.node.surv.fit$surv[elem.node.surv.fit$time == 
					 limit])
      if(is.na(val)) {
	val.low <- min(elem.node.surv.fit$surv[
					       elem.node.surv.fit$time < limit])
	if(is.na(val.low)) {
          #val.low <- 1
	  val.low <- elem.node.surv.fit$surv[1]
	  col.val <- col.bad.right
	}
	val.high <- max(elem.node.surv.fit$surv[
						elem.node.surv.fit$time > limit])
	if(is.na(val.high)) {
	  val.high <- elem.node.surv.fit$surv[length(
						     elem.node.surv.fit$time)]
	  col.val <- col.bad.left
	}
	val <- 0.5 * (val.high + val.low)
      }
      attributes(val)$col <- col.val
      val
    }
  # +++++++++++++++++
  # +++ Action !  +++
  # +++++++++++++++++
  schluss <- F
  ask.flag <- F
  if(missing(limit))
    if(missing(xtile))
      if(interactive == F) {
	stop(NO.ARGUMENTS.STOP)
      }
      else {
        ask.flag <- T
      }
  attr(tree$frame, "row.names") <- as.numeric(row.names(tree$frame))
  number.nodes <- as.numeric(row.names(tree$frame))
  get.ebene <- function(number)
    {
      floor(log(number, 2))
    }
  y.list <- lapply(number.nodes, get.ebene)
  nodes <- as.numeric(row.names(tree$frame))
  parents <- match((nodes %/% 2), nodes)
  nnodes <- dim(tree$frame)[1]
  nnodes.half <- (nnodes - 1)/2
  names(parents) <- 1:nnodes
  sortee <- sort(parents)
  where <- tree$where
  names(where) <- 1:length(where)
  where.list <<- list()
  for(i in as.numeric(names(table(sort(where))))) {
    where.list[[i]] <<- as.numeric(names(where[where == i]))
  }
  flag <- F
  while(flag == F) {
    flag <- T
    for(i in nnodes.half:1) {
      if(is.null(where.list[[as.numeric(names(sortee[2 * 
						     i - 1]))]]) == F) {
	if(is.null(where.list[[as.numeric(names(
						sortee[2 * i]))]]) == F) {
	  where.list[[sortee[2 * i]]] <<- c(
					    where.list[[as.numeric(names(sortee[2 * 
										i - 1]))]], where.list[[as.numeric(
														   names(sortee[2 * i]))]])
	}
        else flag <- F
      }
      else flag <- F
    }
  }
  surv.fit.index <<- function(ii, time, zens)
    {
      surv.fit(time[ii], zens[ii])
    }
  node.surv.fit <<- lapply(where.list, surv.fit.index, time, zens)
  while(schluss == F) {
    if(ask.flag == T) {
      choice.info()
      wahl <- ask.wahl()
      if(wahl == 1) {
	cat(GIVE.LIMIT)
	limit <- ask.limit()
	x.list <- lapply(node.surv.fit, 
			 get.x.rate.surv.fit, limit, col.bad.right,
			 col.bad.left, col.normal)
	x.lab <- paste(STRING.LIMIT.GIVEN, limit)
      }
      else {
        cat(GIVE.XTILE)
        xtile <- ask.xtile(50)/100
        x.list <- lapply(node.surv.fit, 
                         get.x.time.surv.fit, xtile, col.bad.right,
                         col.bad.left, col.normal)
        x.lab <- paste(STRING.XTILE.GIVEN, xtile)
      }
    }
    else {
      if(missing(limit)) {
        x.list <- lapply(node.surv.fit, 
                         get.x.time.surv.fit, xtile, col.bad.right,
                         col.bad.left, col.normal)
        x.lab <- paste(STRING.XTILE.GIVEN, xtile)
      }
      else {
        x.list <- lapply(node.surv.fit, 
                         get.x.rate.surv.fit, limit, col.bad.right,
                         col.bad.left, col.normal)
        x.lab <- paste(STRING.LIMIT.GIVEN, limit)
      }
    }
    if(interactive == T) {
      cat(CHOOSE.DRAW.SPLIT.CRITERIUM)
      text.flag <- ask.wahl()
      cat(CHOOSE.GRAPH.PARAM)
      cat(CHOOSE.ROOTBOX.SIZE)
      rootbox.size <- ask.funct(rootbox.size)
      cat(CHOOSE.HEIGHT.BOX)
      box.height <- ask.funct(box.height)
    }
    xp <- unlist(x.list)
    yp <- unlist(y.list)
    yp <- yp * -1 + max(yp)
    xlp <- list()
    ylp <- list()
    for(i in 1:length(xp)) {
      lenx <- rootbox.size * (tree$frame$n[i]/tree$frame$
			      n[1]) * (max(xp) - min(xp))
      leny <- box.height
      x <- xp[i]
      y <- yp[i]
      xlp[[i]] <- c(x - 0.5 * lenx, x - 0.5 * lenx, x + 
		    0.5 * lenx, x + 0.5 * lenx, x - 0.5 * lenx)
      ylp[[i]] <- c(y - 0.5 * leny, y + 0.5 * leny, y + 
		    0.5 * leny, y - 0.5 * leny, y - 0.5 * leny)
    }
    fatal.list <- list()
    zaehler <- 0
    fun1 <- function(list, value)
      {
	fun2 <- function(elem, value)
	  {
	    elem == value
	  }
	out <- unlist(sapply(list, fun2, value))
	if(length(out[out == T]) == 0)
	  F
        else T
      }
    for(i in 0:max(yp)) {
      ind <- 1:length(xp)
      ind <- ind[yp == i]
      for(j in ind) {
	for(k in ind) {
	  if(j != k) {
	    abstand <- xp[j] - xlp[[j]][1]
	    abstand <- abstand + xp[k] - xlp[[k]][1
						  ]
	    if(abs(xp[j] - xp[k]) <= abstand) {
	      if(zaehler == 0) {
		fatal.list[[1]] <- c(j, k)
		zaehler <- 1
	      }
              else {
                out.j <- unlist(lapply(fatal.list, 
                                       fun1, j))
                out.k <- unlist(lapply(fatal.list, 
                                       fun1, k))
                if(length(out.j[out.j == T]) == 0 && 
                   length(out.k[out.k == T]) == 0) 
                  {
                    zaehler <- 1 + zaehler
                    fatal.list[[zaehler]] <- c(j, k)
                  }
                else {
                  if(length(out.j[out.j == T]) != 0
                     ) {
                    if(length(out.k[out.k == T]) == 
                       0)
                      fatal.list[[out.j == T]] <- c(
                                    fatal.list[[out.j == T]], k
                                    )
                  }
                  if(length(out.k[out.k == T]) != 0
                     ) {
                    if(length(out.j[out.j == T]) == 
                       0)
                      fatal.list[[out.k == T]] <- c(
                                    fatal.list[[out.k == T]], j
                                    )
                  }
                }
              }
	    }
	  }
	}
      }
    }
    if(zaehler != 0) {
      for(i in 1:zaehler) {
	anz.fatal <- length(fatal.list[[i]])
	plusminus <- (-1 * box.height * (anz.fatal - 
					 1))/2
	for(j in 1:anz.fatal) {
	  leny <- box.height
	  yp[fatal.list[[i]][j]] <- yp[fatal.list[[
						   i]][j]] + plusminus
	  y <- yp[fatal.list[[i]][j]]
	  ylp[[fatal.list[[i]][j]]] <- c(y - 0.5 * 
					 leny, y + 0.5 * leny, y + 0.5 * leny, y -
					 0.5 * leny, y - 0.5 * leny)
	  plusminus <- plusminus + box.height
	}
      }
    }
    max.left <- xlp[[1]][1]
    max.right <- xlp[[1]][3]
    for(i in 2:nnodes) {
      if(xlp[[i]][3] > max.right)
	max.right <- xlp[[i]][3]
      if(xlp[[i]][1] < max.left)
	max.left <- xlp[[i]][1]
    }
    plot(xp, yp, pch = ".", xlim = c(max.left - 0.01 * (
			      max.right - max.left), max.right + 0.01 * (
									 max.right - max.left)), ylim = c(-0.01 * max(yp), 
												   ylp[[1]][2] + 0.01 * max(yp)), bty = "n", yaxt = 
	 "n", ylab = "", xlab = x.lab)
    if(node.number.flag == T) {
      if(node.total.no.flag == T) {
	text(xp, yp, paste(row.names(tree$frame), 
			   rep(":", length(tree$frame$n)), tree$
			   frame$n))
      }
      else {
        text(xp, yp, paste(row.names(tree$frame)))
      }
    }
    else {
      text(xp, yp, tree$frame$n)
    }
    for(i in 1:length(xp)) {
      lines(xlp[[i]], ylp[[i]], col = attributes(x.list[[
				  i]])$col)
    }
    text.nodes <- labels(tree)
    for(i in 1:nnodes.half) {
      parent.i <- sortee[2 * i]
      left.i <- as.numeric(names(sortee[2 * i - 1]))
      right.i <- as.numeric(names(sortee[2 * i]))
      if(xp[left.i] > xp[right.i]) {
	dummy <- right.i
	right.i <- left.i
	left.i <- dummy
      }
      p.lenx <- rootbox.size * (tree$frame$n[parent.i]/
				tree$frame$n[1]) * (max(xp) - min(xp))
      leny <- box.height
      l.lenx <- rootbox.size * (tree$frame$n[left.i]/tree$
				frame$n[1]) * (max(xp) - min(xp))
      r.lenx <- rootbox.size * (tree$frame$n[right.i]/
				tree$frame$n[1]) * (max(xp) - min(xp))
      p.x <- xp[parent.i]
      p.y <- yp[parent.i]
      l.x <- xp[left.i]
      l.y <- yp[left.i]
      r.x <- xp[right.i]
      r.y <- yp[right.i]
      xlp <- c(p.x - 0.5 * p.lenx, l.x - 0.5 * l.lenx)
      ylp <- c(p.y - 0.5 * leny, l.y + 0.5 * leny)
      lines(xlp, ylp, col = col.boxconnection, lty = 
	    lty.boxconnection)
      xlp <- c(p.x - 0.5 * p.lenx + l.lenx, l.x + 0.5 * 
	       l.lenx)
      ylp <- c(p.y - 0.5 * leny, l.y + 0.5 * leny)
      lines(xlp, ylp, col = col.boxconnection, lty = 
	    lty.boxconnection)
      if(text.flag == 1) {
	text.x <- xlp[2] + 0.5 * (xlp[1] - xlp[2]) - 
	  0.5 * l.lenx
	text.y <- p.y - 0.5 * leny - 0.55 * (ylp[1] -
					     ylp[2])
	text(text.x, text.y, text.nodes[left.i])
      }
      xlp <- c(p.x + 0.5 * p.lenx, r.x + 0.5 * r.lenx)
      ylp <- c(p.y - 0.5 * leny, r.y + 0.5 * leny)
      lines(xlp, ylp, col = col.boxconnection, lty = 
	    lty.boxconnection)
      xlp <- c(p.x + 0.5 * p.lenx - r.lenx, r.x - 0.5 * 
	       r.lenx)
      ylp <- c(p.y - 0.5 * leny, r.y + 0.5 * leny)
      lines(xlp, ylp, col = col.boxconnection, lty = 
	    lty.boxconnection)
      if(text.flag == 1) {
	text.x <- xlp[2] + 0.5 * (xlp[1] - xlp[2]) + 
	  0.5 * r.lenx
	text.y <- p.y - 0.5 * leny - 0.45 * (ylp[1] -
					     ylp[2])
	text(text.x, text.y, text.nodes[right.i])
      }
    }
    if(interactive == T) {
      cat(DRAW.AGAIN)
      if(ask.wahl() == 0)
	schluss <- T
      else ask.flag <- T
    }
    else schluss <- T
  }
  detach(1)
  dos(paste("deltree /y", tmp.dir))
}

"hist.survtree"<-
function(survtree, ..., nodes, screen.arg = ascr + 1, figs = c(1, length(
	vnames)), plot = T)
{
  bihist <- function(x, y, main = "", brks = NULL, ...)
  {
    if(is.list(x)) {
      y <- x$y
      x <- x$x
    }
    options(warn = 1)
    if(all(is.na(c(x, y)))) {
      par(usr = c(0, 1, 0, 1))
      lines(c(0, 1, NA, 0, 1), c(0, 1, NA, 1, 0))
      warning(paste("Found all NA's at this node for variable", main))
      return()
    }
    if(is.null(brks))
      brks <- hist(c(unclass(x), unclass(y)), plot = F)$
    breaks
    if(all(is.na(x))) {
      ll <- list(counts = numeric(0))
      warning("Left split has all NA's")
    }
    else ll <- hist(unclass(x), breaks = brks, plot = F)
    if(all(is.na(y))) {
      rr <- list(counts = numeric(0))
      warning("Right split has all NA's")
    }
    else rr <- hist(unclass(y), breaks = brks, plot = F)
    xl <- range(c( - ll$counts, rr$counts))
    pn <- par("new")
    if(length(ll$counts) > 0) {
      barplot( - ll$counts, brks, xlim = xl, histo = T, 
              horiz = T, main = main, ...)
      par(new = T)
    }
    if(length(rr$counts) > 0)
      barplot(rr$counts, brks, xlim = xl, histo = T, 
              horiz = T, main = main, ...)
    par(new = pn)
    invisible(list(left = ll$counts, right = rr$counts, breaks = brks))
  }
  survtree.test(survtree)
  vnames <- name.dots()
  if(!length(vnames))
    stop("Must supply variable to inspect")
  if(is.null(data <- data.survtree(survtree)))
    variables <- data.frame(structure(list(...), names = vnames),
                            row.names = database.attr("row.names"),
                            check.names = F)
  else {
    variables <- list()
    for(n in vnames)
      variables[[n]] <- eval(parse(text = n), data)
    variables <- data.frame(variables, row.names = row.names(data),
                            check.names = F)
  }
  where <- survtree$where
  variables <- variables[names(where),  , drop = F]
  frame <- survtree$frame
  node <- as.numeric(row.names(frame))
  leaves <- frame$var == "<leaf>"
  which <- descendants(node)
  maxcnt <- brks <- list()
  for(i in vnames) {
    v <- variables[[i]]
    if(ncat <- length(levels(v))) brks[[i]] <- 0:ncat + 0.5
    else brks[[i]] <- hist(v, plot = F)$breaks
  }
  L <- match(2 * node, node)
  R <- match(2 * node + 1, node)
  split.info <- structure(vector("list", length(vnames)), names = 
			  vnames)
  if(missing(nodes)) {
    if(!(ascr <- screen()))
      stop("Need to invoke tree.screens")
    k <- split.screen(figs, screen = screen.arg)
    screen(ascr, F)
    xy <- treeco(survtree)
    while(length(i <- identify(xy, n = 1, plot = F)) > 0) {
      if(leaves[i]) {
	cat("Terminal node -- try again\n")
	next
      }
      ixl <- which[L[i], where]
      ixr <- which[R[i], where]
      erase.screen(screen.arg)
      i <- 0
      for(j in vnames) {
	split.info[[j]] <- list(x = variables[[j]][ixl],
                                y = variables[[j]][ixr])
	screen(k[i <- i + 1])
	par(mar = c(1, 1, 4.1, 1), cex = 1, err = -1)
	bihist(split.info[[j]], brks = brks[[j]], main = j, axes = F)
      }
      screen(ascr, F)
    }
    on.exit(close.screen(k))
  } else {
    nodes <- node.match(nodes, node, leaves)
    if(length(nodes) == 0)
      return(invisible())
    if(length(nodes) > 1) {
      warning("split information returned on last node only"
              )
      nodes <- nodes[length(nodes)]
    }
    if(plot) {
      # check that we are in split-screen mode
      if(!(ascr <- screen())) stop(
                                   "Need to invoke tree.screens")
      k <- split.screen(figs, screen = screen.arg)
      screen(ascr, F)
    }
    ixl <- which[L[nodes], where]
    ixr <- which[R[nodes], where]
    ii <- 0
    for(j in vnames) {
      split.info[[j]] <- list(x = variables[[j]][ixl], y
                              = variables[[j]][ixr])
      if(plot) {
        # Plot this variable 
        screen(k[ii <- ii + 1])
        par(mar = c(1, 1, 4.1, 1), cex = 1, err = 
            -1)
        bihist(split.info[[j]], brks = brks[[j]], 
               main = j, axes = F)
      }
    }
    if(plot) {
      screen(ascr, F)
      on.exit(close.screen(k))
    }
  }
  if(plot)
    invisible(split.info)
  else split.info
}

"identify.survtree"<-
function(survtree, nodes, pretty = 0, print.it = T)
{
  survtree.test(survtree)
  oc <- sys.call()
  oc[1] <- as.call(as.name("identify.tree"))
  invisible(eval(oc, local = sys.parent()))
}

"labels.survtree"<-
function(object, pretty = T, collapse = T)
{
  survtree.test(object)
  frame <- object$frame
  xlevels <- attr(object, "xlevels")
  var <- frame$var
  splits <- frame$splits
  if(!is.null(pretty)) {
    if(pretty)
      xlevels <- lapply(xlevels, 	#if 0 no abbreviation
			abbreviate, pretty)
    pn <- .C("prlab",
             lab = character(2 * length(var)),
             as.character(splits[, 1]),
             as.character(splits[, 2]),
             as.character(unlist(xlevels)),
             as.integer(unclass(var) - 1),
             #pass an integer with 0 denoting a leaf
             as.integer(sapply(xlevels, length)),
             as.integer(length(var)))
    splits <- matrix(pn$lab, ncol = 2)
  }
  var <- as.character(var)
  if(!collapse)
    return(array(paste(var, splits, sep = ""), dim(splits)))
  node <- as.numeric(row.names(frame))
  parent <- match((node %/% 2), node)
  odd <- as.logical(node %% 2)
  node[odd] <- paste(var[parent[odd]], splits[parent[odd], 2], sep = "")
  node[!odd] <- paste(var[parent[!odd]], splits[parent[!odd], 1], sep = "")
  node[1] <- "root"
  node
}

"labels.survtree.second"<-
function(object, pretty = T, collapse = T)
{
  survtree.second.test(object)
  frame <- object$frame
  xlevels <- attr(object, "xlevels")
  node <- as.numeric(row.names(frame))
  var <- frame$var
  splits <- frame$splits
  if(!is.null(pretty)) {
    if(pretty)
      xlevels <- lapply(xlevels, 	#if 0 no abbreviation
			abbreviate, pretty)
    pn <- .C("prlab",
             lab = character(2 * length(var)),
             as.character(splits[, 1]),
             as.character(splits[, 2]),
             as.character(unlist(xlevels)),
             as.integer(unclass(var) - 1),
             #pass an integer with 0 denoting a leaf
             as.integer(sapply(xlevels, length)),
             as.integer(length(var)))
    splits <- matrix(unlist(lapply(pn$lab, function(x)
                                   {
                                     n <- nchar(x)
                                     if(n > 1) substring(x, 2, n) else ""
                                   }
                                   )), ncol = 2)
  }
  if(!collapse)
    return(splits)
  parent <- match((node %/% 2), node)
  odd <- as.logical(node %% 2)
  node[odd] <- splits[parent[odd], 2]
  node[!odd] <- splits[parent[!odd], 1]
  node[1] <- paste(node[2], ",", node[row.names(frame) == 3], sep = 
                   "")
  node
}

"make.page.screen"<-
  structure(.Data = c(0, 0, 1, 1, 0.97, 0, 1, 0.97), .Dim = c(2, 4))

"make.page.request"<-
function()
{
  par(usr = c(0, 1, 0, 1))
  for(i in 0:4) {
    lines(c(1, 0.9), c(1 - i * 0.05, 1 - i * 0.05))
  }
  for(i in 0:1) {
    lines(c(1 - i * 0.1, 1 - i * 0.1), c(1, 0.8))
  }
  text(rep(0.95, 4), c(0.95, 0.9, 0.85, 0.8) + 0.025,
       c("First", "Previous", "Next", "Last"))
  answer <- F
  i <- 0
  while(answer == F) {
    point <- locator(n = 1)
    if((point$x[1] >= 0.9) && (point$x[1] <= 1) && (point$y[1] <= 1) &&
       (point$y[1] >= 0.8)) {
      answer <- T
      i <- floor((point$y[1] - 0.8) * 20) %% 4
    }
  }
  i
}

"model.frame.survtree"<-
function(formula, ...)
{
  m <- formula$model
  if(!is.null(m)) return(m)
  oc <- formula$call
  if(substring(oc[[1]], 1, 7) == "predict") {
    m <- eval(oc$newdata)
    if(is.null(attr(m, "terms"))) {
      object <- eval(oc$object)
      m <- model.frame(object$terms, m, na.survtree.replace)
    }
    return(m)
  }
  while(oc[[1]] != "survtree") oc <- eval(oc[[2]])$call	
  #	oc$na.action <- NULL #this should work but doesn't
  #	oc$subset <- names(formula$where) #if it did, no need for last expression of fucntion
  oc$subset <- NULL
  oc$method <- "model.frame"
  x <- eval(oc)
  x[names(formula$where),  ]
}

"na.survtree.replace"<-
function(frame)
{
  warning("na.survtree.replace can only execute \"na.omit\". Better exclude NA's before running survtree-functions !\n"
          )
  na.omit(frame)
}

"pages.screens"<- c(1, 2)

"path.survtree"<-
function(survtree, nodes, pretty = 0, print.it = T)
{
  survtree.test(survtree)
  splits <- labels.survtree(survtree, pretty = pretty)
  frame <- survtree$frame
  n <- row.names(frame)
  node <- as.numeric(n)
  which <- descendants(node)            #ancestors are columns
  path <- list()
  if(missing(nodes)) {
    xy <- treeco(survtree)
    while(length(i <- identify(xy, n = 1, plot = F)) > 0) {
      path[[n[i]]] <- path.i <- splits[which[, i]]
      if(print.it) {
        cat("\n", "node number:", n[i], "\n")
        cat(paste("  ", path.i), sep = "\n")
      }
    }
  }
  else {
    if(length(nodes <- node.match(nodes, node)) == 0)
      return(invisible())
    for(i in nodes)
      path[[n[i]]] <- splits[which[, i]]
  }
  invisible(path)
}

"plot.sf.and.st.survtree"<-
function(survtree, time, zens)
{
  survtree.su.fi <- surv.fit(time, zens, survtree$where)
  leaf.names <- row.names(survtree$frame)[as.numeric(names(table(survtree$where)))]
  pos.survtree <- plot(survtree.su.fi)
  pos.survtree$x <- pos.survtree$x + 0.05 * max(survtree.su.fi$time)
  text(pos.survtree, leaf.names)
  plot(survtree)
  survtree$frame$index <- as.numeric(row.names(survtree$frame))
  text(survtree, splits = T, label = "index", pretty = T)
}

"plot.sf.and.st.survtree.second"<-
function(survtree, time, zens)
{
  survtree.second.test(survtree)
  survtree.su.fi <- surv.fit(time, zens, survtree$where)
  leaf.names <- labels(survtree)[as.numeric(names(table(survtree$ where)))]
  pos.survtree <- plot(survtree.su.fi, xlim = c(0, max(survtree.su.fi$ time) *
                                         0.05 + max(survtree.su.fi$time)))
  pos.survtree$x <- pos.survtree$x + 0.02 * max(survtree.su.fi$time)
  text(pos.survtree, labels = leaf.names, adj = 0)
  plot(survtree)
  text(survtree, splits = T, pretty = T)
}

"plot.survtree"<-
function(x, y = NULL, type = NULL, ...)
{
  survtree.test(x)
  call <- match.call()
  call$type <- NULL
  if(is.null(type))
    uniform <- F
  else uniform <- if(length(type) == 1 && type == "u") T else F
  assign(paste(".Tree.unif", dev.cur(), sep = "."), uniform, frame = 0)
  call[[1]] <- as.name("treepl")
  call[["x"]] <- treeco(x)
  call[["y"]] <- as.numeric(row.names(x$frame))
  eval(call)
  invisible()
}

"plot.survtree.sequence"<-
function(x, ..., type = "l", ylim = range(x[["dev"]]))
{
  if(missing(type) && inherits(x, "prune")) type <- "S"
  plot(x[["size"]], x[["dev"]], axes = F, xlab = "size", ylab = 
       "split.complexity", type = type, ylim = ylim, ...)
  box()
  axis(2, ...)
  xaxp <- par("xaxp")
  pos <- seq(xaxp[1], xaxp[2], diff(xaxp[-3])/xaxp[3])
  if(pos[1] == 0) pos[1] <- 1
  n <- length(pos)
  maxsize <- max(x[["size"]])
  if(pos[n] > maxsize) pos[n] <- maxsize
  axis(1, at = pos, pos, ...)
  k <- approx(x[["size"]], x[["k"]], pos)$y
  not.ok <- is.na(k)
  k <- format(signif(k, 2))
  k[not.ok] <- " "
  if(inherits(x, "shrink") & pos[1] == 1) k[1] <- 0
  axis(3, at = pos, lab = k, ...)
  mtext("alpha", line = par()$mgp[1])
  invisible()
}

"predict.survtree"<-
function(object, newdata = list(), zensor.newdata, type = c("vector", "tree"))
{
  survpred <- function(survtree, newdata, zensor.newdata)
  {
    survtree.test(survtree)
    frame <- survtree$frame
    node <- as.numeric(row.names(frame))
    ndim <- ceiling(length(node)/2)
    if(is.null(attr(newdata, "terms")))
      newdata <- model.frame(survtree$terms, newdata,
                             na.action = na.survtree.replace)
    where <- pred.tree(survtree, data.matrix(newdata))
    y <- model.extract(newdata, "response")
    zp <- .C("newpred",
             as.integer(length(node)),
             as.integer(node),
             as.double(y),
             as.double(zensor.newdata),
             as.double(where),
             as.integer(length(y)),
             list(pchisq),
             log.rank.value = double(length(node)),
             log.rank.alpha = double(length(node)),
             deviance = double(length(node)))
    retur <- list()
    retur$log.rank.alpha <- zp$log.rank.alpha
    retur$log.rank.value <- zp$log.rank.value
    retur$dev <- zp$deviance
    retur
  }
  survtree.test(object)
  if(missing(type)) type <- "vector"
  else type <- match.arg(type, c("vector", "tree"))
  if(missing(newdata) & type == "tree")
    return(object)			#idiot proofing
  if(missing(newdata)) where <- object$where
  else {
    if(is.null(attr(newdata, "terms"))) {
      Terms <- object$terms
      a <- attributes(Terms)
      y <- match(as.character(a$variables[a$response]), names(newdata))
      if(is.na(y)) Terms <- delete.response(Terms)
      newdata <- model.frame(Terms, newdata, na.action = 
                             na.survtree.replace)
    }
    where <- pred.tree(object, data.matrix(newdata))
  }
  frame <- object$frame
  if(type != "tree") return(where)
  which <- descendants(as.numeric(row.names(frame)))[, where]
  if(is.na(y)) {
    dev <- rep(NA, nrow(frame))
    log.rank.value <- rep(NA, nrow(frame))
    log.rank.alpha <- rep(NA, nrow(frame))
  } else {
    new.dat <- survpred(object, newdata, zensor.newdata)
    dev <- new.dat$dev
    log.rank.value <- new.dat$log.rank.value
    log.rank.alpha <- new.dat$log.rank.alpha
  }
  object$frame$dev <- as.vector(dev)
  object$frame$log.rank.value <- as.vector(log.rank.value)
  object$frame$log.rank.alpha <- as.vector(log.rank.alpha)
  object$frame$n <- as.vector(which %*% rep(1, length(where)))
  object$where <- where
  object$call <- match.call()
  object$y <- y
  object
}

"print.survtree"<-
function(x, pretty = 0, spaces = 2, digits = .Options$digits - 3, ...)
{
  survtree.test(x)
  cat("node), split, n, log.rank.value\n")
  cat("      * denotes terminal node\n\n")
  frame <- x$frame
  node <- as.numeric(row.names(frame))
  depth <- tree.depth(node)
  indent <- paste(rep(" ", spaces * 32), collapse = "")	
					#32 is the maximal depth
  indent <- substring(indent, 1, spaces * seq(depth))
  indent <- paste(c("", indent[depth]), format(node), ")", sep = "")
  log.r <- format(signif(frame$log.rank.value, digits = digits))
  term <- rep(" ", length(depth))
  term[frame$var == "<leaf>"] <- "*"
  z <- labels(x, pretty = pretty)
  z <- paste(indent, z, frame$n, log.r, term)
  cat(z, sep = "\n")
  invisible(x)
}

"prune.survtree"<-
function(survtree, k, newdata, zensor.newdata, fest.alpha = 0)
{
  survtree.test(survtree)
  if(!missing(k) && any(k) < 0) stop("Argument k must be > 0 !\n")
  frame <- survtree$frame
  node <- row.names(frame)
  ndim <- length(node)
  if(!missing(k))
    ndim <- max(length(k), ndim)
  where <- survtree$where
  if(missing(zensor.newdata))
    zensor.newdata <- rep(0, length(where))
  if(!missing(newdata)) {
    if(is.null(attr(newdata, "terms")))
      newdata <- model.frame(survtree$terms, newdata, 
			     na.action = na.survtree.replace)
    where <- pred.tree(survtree, data.matrix(newdata))
    y <- model.extract(newdata, "response")
  } else where <- y <- NULL
  if(nylevels <- length(attr(survtree, "ylevels")))
    yval <- t(frame$yprob)
  else yval <- frame$yval
  zp <- .C("newprune",
	   as.integer(length(node)),
	   as.integer(node),
	   as.double(frame$log.rank.value),
	   as.double(y),
	   as.double(zensor.newdata),
	   as.double(where),
	   as.integer(length(y)),
	   as.double(fest.alpha),
	   deviance = double(ndim),
	   newdev = double(ndim),
	   alpha = double(ndim),
	   size = integer(ndim),
	   n = integer(1),
	   inode = integer(ndim))	
  #		as.integer(frame$var != "<leaf>"), 
  if(!missing(k) & !missing(newdata)) {
    n <- length(k)
    alpha <- k
  } else {
    n <- zp$n
    alpha <- zp$alpha[1:n]
  }
  size <- zp$size[1:n]
  deviance <- if(missing(newdata)) zp$deviance[1:n] else zp$newdev[1: n]
  if(!missing(k) & missing(newdata)) {
    nodes <- c(0, node[zp$inode])
    k <- sum(max(k) > alpha)
    if(k > 1) {
      survtree <- snip.survtree(survtree, nodes[2:k])
      survtree$call <- match.call()
    }
    return(survtree)
  } else {
    size <- size - 1
    deviance <- deviance - alpha * size
    obj <- list(size = size, dev = deviance, k = alpha)
    class(obj) <- c("prune", "survtree.sequence")
    return(obj)
  }
}

"select.survtree"<-
function(survtree, nodes, drop = T)
{
  survtree.test(survtree)
  oc <- sys.call()
  oc[1] <- as.call(as.name("select.tree"))
  eval(oc, local = sys.parent())
}

"snip.survtree"<-
function(survtree, nodes, xy.save = F, digits = .Options$digits - 3)
{
  survtree.test(survtree)
  node <- as.numeric(row.names(survtree$frame))
  dev <- survtree$frame$dev
  j <- node.match(nodes, node, survtree$frame$var == "<leaf>")
  if(length(j) != 0) {
    nodes <- node[j]
    which <- descendants(node, include = T)
    nodes <- nodes[apply(which[j,  ] %*% which[, j], 2, "sum") == 1]
    for(i in nodes) {
      curnode <- i
      if(length(dev[node == curnode]) != 0) {
	devminus <- dev[node == curnode]
	parent <- match((curnode %/% 2), node) + 1
	while(parent != 1) {
	  parent <- match((curnode %/% 2), node)
	  dev[parent] <- dev[parent] - devminus
	  curnode <- node[parent]
	}
      }
    }
  }
  survtree$frame$dev <- dev		#class.old <- class(survtree)
  frame <- survtree$frame
  attr(frame, "row.names") <- as.numeric(row.names(frame))
  survtree$frame <- frame
  if(missing(nodes))
    res <- snip.tree(survtree, xy.save = xy.save, digits = digits)
  else res <- snip.tree(survtree, nodes, xy.save = xy.save, digits = digits)
  frame <- res$frame
  row.names(frame) <- row.names(frame)
  res$frame <- frame
  res$frame$dev <- ifelse(res$frame$var == "<leaf>",
                          rep(0, dim(res$ frame)[1]), res$frame$dev)
  res$frame$log.rank.value <- ifelse(res$frame$var == "<leaf>",
                                     rep(0, dim(res$frame)[1]),
                                     res$frame$log.rank.value)
  res$frame$log.rank.alpha <- ifelse(res$frame$var == "<leaf>",
                                     rep(0, dim(res$frame)[1]),
                                     res$frame$log.rank.alpha)
  res
}

"survcart.demo"<-
function(real.time = F, FUN = win.graph, geometry="", 
	cex.great = 0.90, cex.middle = 0.78, cex.small = 0.5, ...)
{
  make.page.request.start <- 
    function(place, par.usr, width.menu, 
	     page.nr, first.page.nr, last.page.nr, cex.great, cex.middle,
	     cex.small)
  {
    number.no <- last.page.nr - first.page.nr + 1
    box.w.menu <- ((place$right - place$left) * width.menu)/5
    box.h.menu <- (place$top - place$bottom)
    box.w.numbers <- ((place$right - place$left) * (1 - width.menu))/number.no
    box.h.numbers <- box.h.menu
    border.w <- (place$right - place$left) * width.menu + place$left
    box.adj.w.menu <- box.w.menu/2
    box.adj.w.numbers <- box.w.numbers/2
    box.adj.h <- box.h.menu/2
    par(usr = par.usr)
    for(i in 0:5) {
      lines(c(rep(border.w - (i * box.w.menu), 2)),
            c(place$top, place$bottom))
    }
    for(i in 1:number.no) {
      lines(c(rep(border.w + (i * box.w.numbers), 2)),
            c(place$top, place$bottom))
    }
    for(i in 0:1) {
      lines(c(place$left, place$right),
            rep(place$top - i * (box.h.menu), 2))
    }
    text(rep(border.w, 5) - rep(box.adj.w.menu, 5) - c(0:4) *
         box.w.menu, rep(place$bottom + box.adj.h, 5),
         c("Last", "Next", "Previous", "First", "Quit"), cex = cex.middle)
#    text(rep(border.w, number.no) + rep(box.adj.w.numbers, number.no) +
#         c(0:(last.page.nr - first.page.nr)) * box.w.numbers,
#         rep(place$bottom + box.adj.h, number.no),
#         as.character(first.page.nr:last.page.nr), cex = cex.small)
#    text(border.w + box.adj.w.numbers + (page.nr - first.page.nr) *
#         box.w.numbers, rep(place$bottom + box.adj.h, number.no),
#         as.character(page.nr), col = 0, cex = cex.small)
    text(rep(border.w, number.no-1) + rep(box.adj.w.numbers, number.no-1) +
         c(0:(last.page.nr - first.page.nr))[-(page.nr - first.page.nr+1)] *
         box.w.numbers,
         rep(place$bottom + box.adj.h, number.no-1),
         as.character(first.page.nr:last.page.nr)[-(page.nr - first.page.nr+1)], cex = cex.small)
    text(border.w + box.adj.w.numbers + (page.nr - first.page.nr) *
         box.w.numbers, rep(place$bottom + box.adj.h, number.no),
         as.character(page.nr), cex = cex.middle)
  }
  make.page.request <- function(place, par.usr, width.menu, page.nr, 
				first.page.nr, last.page.nr, cex.great,
                                cex.middle, cex.small)
  {
    number.no <- last.page.nr - first.page.nr + 1
    box.w.menu <- ((place$right - place$left) * width.menu)/5
    box.h.menu <- (place$top - place$bottom)
    box.w.numbers <- ((place$right - place$left) * (1 - width.menu))/number.no
    box.h.numbers <- box.h.menu
    border.w <- (place$right - place$left) * width.menu + place$left
    box.adj.w.menu <- box.w.menu/2
    box.adj.w.numbers <- box.w.numbers/2
    box.adj.h <- box.h.menu/2
    par(usr = par.usr)
    answer <- F
    i <- 0
    while(answer == F) {
      point <- locator(n = 1)
      if((point$x[1] >= place$left) && (point$x[1] <= place$right)
         && (point$y[1] <= place$top) && (point$y[1] >= place$bottom)) {
        if(point$x[1] <= border.w) {
          i <- floor((point$x[1] - place$left)/ box.w.menu)
          if((i == 0) || ((i != 1 || page.nr != first.page.nr) &&
                (i != 2 || page.nr != first.page.nr) &&
                (i != 3 || page.nr != last.page.nr) &&
                (i != 4 || page.nr != last.page.nr))) {
            answer <- T
            if(i == 0) ret.val <- -99
            if(i == 1) ret.val <- first.page.nr
            if(i == 2) ret.val <- page.nr - 1
            if(i == 3) ret.val <- page.nr + 1
            if(i == 4) ret.val <- last.page.nr
          }
        } else {
          i <- floor((point$x[1] - border.w)/box.w.numbers)
          i <- i + first.page.nr
          if(page.nr != i) {
            answer <- T
            ret.val <- i
          }
        }
      }
    }
    frame()
    split.screen(make.page.screen)
    screen(1)
    par(usr = par.usr)
      for(i in 0:5) {
        lines(c(rep(border.w - (i * box.w.menu), 2)),
              c(place$top, place$bottom))
      }
      for(i in 1:number.no) {
        lines(c(rep(border.w + (i * box.w.numbers), 2)),
              c(place$top, place$bottom))
      }
      for(i in 0:1) {
        lines(c(place$left, place$right),
              rep(place$top - i * (box.h.menu), 2))
      }
    text(rep(border.w, 5) - rep(box.adj.w.menu, 5) - c(0:4) *
         box.w.menu, rep(place$bottom + box.adj.h, 5),
         c("Last", "Next", "Previous", "First", "Quit"), cex = cex.middle)
    text(rep(border.w, number.no-1) + rep(box.adj.w.numbers, number.no-1) +
         c(0:(last.page.nr - first.page.nr))[-(ret.val - first.page.nr+1)] *
         box.w.numbers,
         rep(place$bottom + box.adj.h, number.no-1),
         as.character(first.page.nr:last.page.nr)[-(ret.val - first.page.nr+1)], cex = cex.small)
    text(border.w + box.adj.w.numbers + (ret.val - first.page.nr) *
         box.w.numbers, rep(place$bottom + box.adj.h, number.no),
         as.character(ret.val), cex = cex.middle, col=2)
    ret.val
  }
  assign("cex.great", cex.great, frame = 1)
  my.vu <- function(text, cex.great = substitute(cex.great, 1))
  {
    text <- c(".L 4", text)
    vu(text, wid = par()$fin[1] - 0.1 * par()$fin[1],
       hei = par()$fin[2] - 0.1 * par()$fin[2],
       csize = c(cex.great)
       )
  }
  erase.screen <- function(...){}
  # +++ ++++++++++++++++++++++++++++++++++++++++++ +++ #
  # +++ 1. Schritt                                 +++ #
  # +++ das Ueberlebenszeit-Baumobjekt "dworak.st" +++ #
  # +++ wird erzeugt.                              +++ #
  # +++ ++++++++++++++++++++++++++++++++++++++++++ +++ #
  # +++ ein X11-Fenster wird erzeugt;              +++ #
  # +++ in dieses wird in voller Groesse der Baum  +++ #
  # +++ dworak.st gezeichnet. Bei jedem Knoten     +++ #
  # +++ wird die Anzahl der Faelle in diesem       +++ #
  # +++ Knoten mit dazu geschrieben.               +++ #
  # +++ ++++++++++++++++++++++++++++++++++++++++++ +++ #
  page.titlepage <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.titlepage)
    }
  page.contents <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.contents.1)
      screen(cur.screens[2])
      my.vu(vu.contents.2)
    }
  page.introduction <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.introduction.1)
      screen(cur.screens[2])
      my.vu(vu.introduction.2)
    }
  page.implementation <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.implementation.1)
      screen(cur.screens[2])
      my.vu(vu.implementation.2)
    }
  page.datademo <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.datademo.1)
      screen(cur.screens[2])
      my.vu(vu.datademo.2)
      screen(cur.screens[3])
      my.vu(vu.datademo.3)
      screen(cur.screens[4])
      my.vu(vu.datademo.4)
    }
  page.datademo.continue <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.datademo.5)
      screen(cur.screens[2])
      my.vu(vu.datademo.6)
    }
  page.LM.tree <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.analysis.1)
      screen(cur.screens[2])
      my.vu(vu.analysis.2)
    }
  page.strategy <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.strategy.1)
      screen(cur.screens[2])
      my.vu(vu.strategy.2)
    }
  page.prune.sequence <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.prune)
    }
  page.grouping <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.grouping.1)
      screen(cur.screens[2])
      my.vu(vu.grouping.2)
    }
  page.interest <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.interest.1)
    }
  page.ongoing <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.ongoing.1)
    }
  page.finally <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.finally.1)
    }
  page.references <- function(cur.screens, my.vu)
    {
      screen(cur.screens[1])
      my.vu(vu.references)
    }
  if(!exists("LM.prune")) {
    cat("Please wait: I have to initialize some objects for the\ndemonstration. Don't panic.\n"
	)
    survcart.demo.init()
  }
  FUN(...)
  cat("Re-size and position the graphics window, then hit Return")
  readline()
  split.screen(make.page.screen)	#	par(cex = 1)
  screen(2)
  pages.screens <- split.screen(c(1, 2))#	par(cex = 1)
  page.titlepage(2, my.vu)
  screen(1)
  place <- list()
  place$top <- 0.9
  place$bottom <- 0.1
  place$right <- 0.95
  place$left <- 0.05
  par.usr <- c(0, 1, 0, 1)
  width.menu <- 0.5
  page.nr <- -1
  first.page.nr <- -1
  last.page.nr <- 16
  make.page.request.start(place, par.usr, width.menu, page.nr, 
			  first.page.nr, last.page.nr, cex.great, cex.middle, 
			  cex.small)
  answer <- 1
  while(answer != -99) {
    screen(1, new = F)
    answer <- make.page.request(place, par.usr, width.menu, 
				page.nr, first.page.nr, last.page.nr, cex.great, 
				cex.middle, cex.small)
    if(page.nr == -1) {
      erase.screen(2)
    }
    if(page.nr == 0) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 1) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 2) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 3) {
      for(i in c(cur.screens1, cur.screens2)) {
	erase.screen(i)
      }
      screen(2)
      close.screen(c(cur.screens1, cur.screens2))
    }
    if(page.nr == 4) {
      for(i in cur.screens3) {
	erase.screen(i)
      }
      screen(2)
      close.screen(cur.screens3)
    }
    if(page.nr == 5) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 6) {
      erase.screen(2)
    }
    if(page.nr == 7) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 8) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 9) {
      for(i in cur.screens4) {
	erase.screen(i)
      }
      screen(2)
      close.screen(cur.screens4)
    }
    if(page.nr == 10) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 11) {
      for(i in cur.screens5) {
	erase.screen(i)
      }
      screen(2)
      close.screen(cur.screens5)
    }
    if(page.nr == 12) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 13) {
      for(i in cur.screens6) {
	erase.screen(i)
      }
      screen(2)
      close.screen(cur.screens6)
    }
    if(page.nr == 14) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 15) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(page.nr == 16) {
      erase.screen(pages.screens[1])
      erase.screen(pages.screens[2])
    }
    if(answer != -99) {
      page.nr <- answer
      if(page.nr == -1) {
	page.titlepage(2, my.vu)
      }
      if(page.nr == 0) {
	page.contents(pages.screens, my.vu)
      }
      if(page.nr == 1) {
	page.introduction(pages.screens, my.vu)
      }
      if(page.nr == 2) {
	page.implementation(pages.screens, my.vu)
      }
      if(page.nr == 3) {
	screen(pages.screens[1])
	cur.screens1 <- split.screen(
				     datademo.screen)
	page.datademo(cur.screens1, my.vu)
	screen(pages.screens[2])
	cur.screens2 <- split.screen(c(2, 1))
	screen(cur.screens2[1])
	cex.old <- par()$cex
	par(cex = cex.middle)
	cex.factor <- cex.middle/cex.old
	dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor	
        #				par(mgp = c(5, 2, 0))
	par(mai = dummy.mai)
	hist(LM$age, xlab = "age: Age in years")
	title("Age of patient")
	screen(cur.screens2[2])
	cex.old <- par()$cex
	par(cex = cex.middle)
	cex.factor <- cex.middle/cex.old
	dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor	
        #				par(mgp = c(5, 2, 0))
	par(mai = dummy.mai)
	hist(LM$dm, xlab = "dm: diameter in cm")
	title("Diameter of largest LM")
      }
      if(page.nr == 4) {
	screen(2)
	cur.screens3 <- split.screen(barplot.screen
				     )
	page.datademo.continue(cur.screens3, my.vu)
	screen(cur.screens3[3])
	mm <- matrix(rep(0, 7 * 4), ncol = 4, 
		     dimnames = list(names(LM)[c(1, 2, 3, 5, 6, 8, 9)], NULL))
	for(i in c(1, 2, 3, 5, 6, 8, 9)) {
	  xx <- table(LM[, i])
	  for(j in 1:4) {
	    if(!is.na(xx[j])) {
	      mm[dimnames(mm)[[1]] == names(LM)[i], j] <- xx[j]
	    }
	  }
	}
	par(cex = cex.middle)
	barplot(t(mm), angle = c(45, 135), density = c(1:4) * 6,
                names = dimnames(mm)[[1]], horiz = T)
      }
      if(page.nr == 5) {
	page.LM.tree(pages.screens, my.vu)
      }
      if(page.nr == 6) {
	screen(2)
	plot(LM.tree)
	par(cex = cex.middle)
	text(LM.tree, pretty = T)
      }
      if(page.nr == 7) {
	page.strategy(pages.screens, my.vu)
      }
      if(page.nr == 8) {
	page.prune.sequence(pages.screens, my.vu)
	screen(pages.screens[2])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	par.mai.old <- par()$mai
	dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[2] <- dummy.mai[2] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor
	dummy.mai[4] <- dummy.mai[4] * cex.factor
	par(mai = dummy.mai)
	par(mgp = c(5, 2, 0))
	plot(LM.prune)
	par(mai = par.mai.old)
	par(cex = cex.old)
      }
      if(page.nr == 9) {
	screen(2)
	cur.screens4 <- split.screen(c(2, 3))
	for(i in 1:6) {
	  screen(cur.screens4[i])
	  cex.old <- par()$cex
	  cex.factor <- cex.middle/cex.old
	  par(cex = cex.middle)
	  dummy.mai <- par()$mai
	  dummy.mai[1] <- dummy.mai[1] * cex.factor
	  dummy.mai[3] <- dummy.mai[3] * cex.factor
          #				  par(mgp = c(5, 2, 0))
	  par(mai = dummy.mai)
	  plot.survtree(LM.prune.extract[[i]])	
          #			text(LM.prune.extract[[i]], label = NULL, pretty = T, cex = 1.3)
	  text(LM.prune.extract[[i]], label = NULL, 
	       pretty = T)
	  tit <- paste("alpha =", signif(LM.prune$k[55 + i], 4))
	  title(tit)
	}
      }
      if(page.nr == 10) {
	page.grouping(pages.screens, my.vu)
      }
      if(page.nr == 11) {
	screen(2)
	cur.screens5 <- split.screen(c(3, 4))
	for(i in 1:12) {
	  screen(cur.screens5[i])
	  cex.old <- par()$cex
	  cex.factor <- cex.middle/cex.old
	  par(cex = cex.middle)
	  dummy.mai <- par()$mai
	  dummy.mai[1] <- dummy.mai[1] * cex.factor
	  dummy.mai[2] <- dummy.mai[2] * cex.factor
          #				  dummy.mai[3] <- dummy.mai[3] * cex.factor
          #				  dummy.mai[4] <- dummy.mai[4] * cex.factor	
	  par(mgp = c(5, 2, 0))
	  par(mai = dummy.mai)
	  plot(LM.grouping.sv[[i + 8]], mark.time
	       = F)
	  tit <- paste("alpha =", signif(LM.prune$k[40 + 8 + i], 4))
	  title(tit)
	}
      }
      if(page.nr == 12) {
	page.interest(pages.screens, my.vu)
	screen(pages.screens[2])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	par.mai.old <- dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor
	par(mgp = c(5, 2, 0))
	par(mai = dummy.mai)
	graph.survtree(LM.tr.5.1, LM$t, LM$z, xtile = 0.5,
                       node.total.no.flag = F, inter = F)
	par(mai = par.mai.old)
	par(cex = cex.old)
      }
      if(page.nr == 13) {
	page.ongoing(pages.screens[1], my.vu)
	screen(pages.screens[2])
	cur.screens6 <- split.screen(c(2, 1))
	screen(cur.screens6[2])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[2] <- dummy.mai[2] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * 4
	par(mai = dummy.mai)
	par(mgp = c(5, 2, 0))
	plot(LM.tr.5.1.sec.prune)
	screen(cur.screens6[1])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor
	par(mai = dummy.mai)
	par(mgp = c(5, 2, 0))
	plot(LM.tr.5.1.sec)
	text(LM.tr.5.1.sec, pretty = T)
      }
      if(page.nr == 14) {
	page.finally(pages.screens, my.vu)
      }
      if(page.nr == 15) {
	screen(pages.screens[1])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	par.mai.old <- dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[2] <- dummy.mai[2] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor
	par(mgp = c(5, 2, 0))
	par(mai = dummy.mai)
	survtree.su.fi <- surv.fit(LM$t, LM$z, 
				   LM.stage.3$where)
	leaf.names <- labels(LM.stage.3)[as.numeric(names(table(LM.stage.3$where)))]
	pos.survtree <- plot(survtree.su.fi,
                             xlim = c(0, max(survtree.su.fi$time) * 0.05 +
                               max(survtree.su.fi$time)))
	pos.survtree$x <- pos.survtree$x + 0.02 * max(survtree.su.fi$time)
	text(pos.survtree, labels = leaf.names, adj = 0)
	par(mai = par.mai.old)
	par(cex = cex.old)
	screen(pages.screens[2])
	cex.old <- par()$cex
	cex.factor <- cex.middle/cex.old
	par(cex = cex.middle)
	par.mai.old <- dummy.mai <- par()$mai
	dummy.mai[1] <- dummy.mai[1] * cex.factor
	dummy.mai[3] <- dummy.mai[3] * cex.factor
	par(mgp = c(5, 2, 0))
	par(mai = dummy.mai)
	graph.survtree(LM.tr.5.1, LM$t, LM$z, xtile = 0.5, inter = F)	
        #				par(mai = par.mai.old)
	par(mai = par.mai.old)
	par(cex = cex.old)
      }
      if(page.nr == 16) {
	page.references(pages.screens, my.vu)
      }
    }
  }
  close.screen(all = T)
  dev.off()
}

"survcart.demo.init"<-
function()
{
  LM.prune <<- prune.survtree(LM.tree)
  LM.prune.extract <<- list()
  for(i in 1:6) {
    LM.prune.extract[[i]] <<- prune.survtree(LM.tree, k = LM.prune$k[55 + i])
  }
  LM.tree.list <<- list()
  LM.tree.list[[1]] <<- prune.survtree(LM.tree, k = LM.prune$k[40 + 1])
  for(i in 2:20) {
    LM.tree.list[[i]] <<- prune.survtree(LM.tree.list[[i - 1]], 
					 k = LM.prune$k[40 + i])
  }
  LM.grouping.sv <<- list()
  for(i in 1:20) {
    LM.grouping.sv[[i]] <<- surv.fit(LM$t, LM$z, LM.tree.list[[i]]$where)
  }
  LM.tr.5.1 <<- prune.survtree(LM.tree, k = 5.1)
  LM.tr.5.1.sec <<- survtree.second(LM.tr.5.1, LM$t, LM$z)
  LM.tr.5.1.sec.prune <<- prune.survtree(LM.tr.5.1.sec)
  LM.stage.3 <<- prune.survtree(LM.tr.5.1.sec, k = 15)
  invisible(T)
}

"survtree"<-
function(formula = formula(data), data = sys.parent(), zensor, fact.flag, 
	ask.flag = 0, left.t = 0.1, right.t = 0.9, file, weights, subset, 
	na.action = na.survtree.replace, control = tree.control(nobs, ...), 
	method = "recursive.partition", model = NULL, x = F, y = T, z = T, 
	...)
{
  NO.ZENSOR.STOP <- "No argument zensor given !\n"
  NO.FACT.FLAG.STOP <- "No argument fact.flag given !\n"
  NO.BINARY.ZENSOR.STOP <- "Argument zensor must be binary !\n"
  NO.01.ZENSOR.STOP <- 
    "Argument zensor must be binary with values 0 ( = \"no death observed\") and 1 ( = \"death observed\") !\n"
  NO.EQUAL.LENGTH.STOP <- 
    "Argument zensor must have equal length as argument \"survival-time\" !\n"
  NO.RIGHT.FACT.FLAG.STOP <- 
    "Length of argument fact.flag must be equal to number of independent variables (see help(survtree) for details on fact.flag) !\n"
  NO.LOGICAL.FACT.FLAG.STOP <- 
    "Argument fact.flag must be vector of logicals !\n"
  my.readline <- function()
    {
      ret <- -3
      x <- readline()
      if(is.na(as.numeric(x))) {
	if(nchar(x) >= 1) {
	  x <- substring(x, 1, 1)
	  if(x == "!") ret <- -2
	  if(x == "?") ret <- -1
	}
      }
      else {
        x <- as.numeric(x)
        if(floor(x) == x) ret <- x
      }
      storage.mode(ret) <- "integer"
      ret
    }
  if(missing(fact.flag)) stop(NO.FACT.FLAG.STOP)
  if(missing(zensor)) stop(NO.ZENSOR.STOP)	
  #if(version$major == 3 && version$minor == 2) {
  if(is.null(model)) {
    model.all <- match.call(expand = F)
    model <- model.all[1]
    mode(model) <- "call"
    if(!is.null(model.all$formula)) {
      model$formula <- model.all$formula
    }
    if(!is.null(model.all$data)) {
      model$data <- model.all$data
    }
    model[length(model) + 1] <- model.all[names(model.all) == "zensor"]
    mode(model.all) <- "list"
    if(length(model.all[names(model.all) == "na.action"]) != 0
       ) {
      model[length(model) + 1] <- model.all[names(model.all) == "na.action"]
    }
    if(length(model.all[names(model.all) == "subset"]) != 0) {
      model[length(model) + 1] <- model.all[names(model.all) == "subset"]
    }
    mode(model.all) <- "call"
    model[length(model) + 1] <- model.all[length(model.all)]
    model$method <- model$model <- model$control <- model$... <- 
      NULL
    model[[1]] <- as.name("model.frame.default")
    mode(model) <- "call"
    model <- eval(model, sys.parent())
    if(method == "model.frame") return(model)
  }
  #}
  #else {
  #		if(is.null(model)) {
  #			model <- match.call(expand = F)
  #			model$method <- model$model <- model$control <- model$
  #				... <- NULL
  #			model[[1]] <- as.name("model.frame.default")
  #			model <- eval(model, sys.parent())
  #			if(method == "model.frame")
  #				return(model)
  #		}
  #	}
  zensor <- model$"(zensor)"
  if(missing(file)) {
    log.flag <- F
    file <- ""
  }
  else log.flag <- T
  Terms <- attr(model, "terms")
  if(any(attr(Terms, "order") > 1))
    stop("Survtrees cannot handle interaction terms")
  Y <- model.extract(model, "response")
  w <- model.extract(model, "weights")
  if(!length(w)) w <- rep(1, nrow(model))
  X <- data.matrix(model)
  xlevels <- attr(X, "column.levels")
  if(is.null(xlevels)) {
    xlevels <- rep(list(NULL), ncol(X))
    names(xlevels) <- dimnames(X)[[2]]
  }
  nobs <- length(Y)
  tab.zensor <- table(zensor)
  if(length(tab.zensor) != 2) {
    if(length(tab.zensor) != 1) {
      stop(NO.BINARY.ZENSOR.STOP)
    } else {
      if(names(tab.zensor)[1] != "1") stop(NO.01.ZENSOR.STOP)
    }
  } else {
    if((names(tab.zensor)[1] != "0") | (names(tab.zensor)[2] != "1"))
      stop(NO.01.ZENSOR.STOP)
  }
  if(length(zensor) != nobs) stop(NO.EQUAL.LENGTH.STOP)
  if(length(fact.flag) != length(xlevels)) stop(NO.RIGHT.FACT.FLAG.STOP)
  if(mode(fact.flag) != "logical") stop(NO.LOGICAL.FACT.FLAG.STOP)
  fit <- .C("newgrow",
	    as.double(X),
	    as.double(unclass(Y)),
	    as.double(w),
	    as.integer(c(sapply(xlevels, length))),
	    as.character(dimnames(X)[[2]]),
	    as.character(unlist(xlevels)),
	    as.integer(zensor),
	    as.integer(nobs),
	    as.integer(ncol(X)),
	    as.integer(ask.flag),
	    as.integer(log.flag),
	    as.character(file),
	    node = integer(control$nmax),
	    var = integer(control$nmax),
	    cutleft = character(control$nmax),
	    cutright = character(control$nmax),
	    n = integer(control$nmax),
	    dev = double(control$nmax),
	    log.rank.value = double(control$nmax),
	    log.rank.alpha = double(control$nmax),
	    as.integer(control$minsize),
	    as.integer(control$mincut),
	    as.double(left.t),
	    as.double(right.t),
	    list(pchisq),
	    list(qchisq),
	    list(my.readline),
	    as.integer(fact.flag),
	    nnode = integer(1),
	    where = integer(nobs))
  n <- fit$nnode
  frame <- data.frame(fit[c("var", "n", "dev", "log.rank.value", 
			    "log.rank.alpha")])[1:n,  ]
  frame$var <- factor(frame$var, 0:length(xlevels), c("<leaf>", names(xlevels)))
  frame$splits <- array(unlist(fit[c("cutleft", "cutright")]), c(control$nmax, 2), list(character(0), c("cutleft", "cutright")))[1:n,  ]
  row.names(frame) <- fit$node[1:n]
  where <- match(fit$where, row.names(frame))
  if(version$major == 3 && version$minor == 2) {
    row.names(frame) <- fit$node[1:n]
  } else {
    attr(frame, "row.names") <- fit$node[1:n]
  }
  fit <- list(frame = frame, where = where, terms = Terms, call = match.call())
  attr(fit$where, "names") <- row.names(model)
  attr(fit, "class") <- c("survtree", "tree")
  attr(fit, "xlevels") <- xlevels
  attr(fit, "left.t") <- left.t
  attr(fit, "right.t") <- right.t
  if(x) fit$x <- X
  if(y) fit$y <- Y
  if(z) fit$z <- zensor
  fit
}

"survtree.second"<-
function(object, time, zensor)
{
  survtree.test(object)
  frame <- object$frame
  where <- as.factor(row.names(frame)[object$where])
  zens <- length(zensor)
  zensor <- zensor
  data <- list()
  data$zens <- zensor
  data$time <- time
  data$where <- where
  assign("data", data.frame(data), frame = 1)
  second <- survtree(time ~ where, data = data, zens = data$zens, fac
                     = T, ask.flag = 0)
  class(second) <- c("survtree.second", class(object))
  second
}

"survtree.second.test"<-
function(x)
{
  if(!inherits(x, "survtree.second"))
    stop("No object of class survtree.second given !\n")
}

"survtree.test"<-
function(x)
{
  if(!inherits(x, "survtree"))
    stop("No object of class survtree given !\n")
}

"text.survtree"<-
function(x, splits = T, label = "n", FUN = text, all = T, pretty = NULL, 
	digits = .Options$digits - 3, ...)
{
  #Put left splits at the parent node
  #Allow labels at all or just leaf nodes
  #if FUN=text then adj=1 puts the split label to the left of the split rather than centered
  if(!inherits(x, "tree")) stop("Not legitimate tree")
  frame <- x$frame
  col <- names(frame)
  if(!is.null(ylevels <- attr(x, "ylevels")))
    col <- c(col, ylevels)
  if(!is.null(label)) {
    if(is.na(match(label, col)))
      stop("Label must be a column label of the frame component of the tree")
  }
  cxy <- par("cxy")[2]
  if(!is.null(srt <- list(...)$srt) && srt == 90) cxy <- cxy * 4
  xy <- treeco(x)
  if(splits) {
    node <- as.numeric(row.names(frame))
    left.child <- match(2 * node, node)
    rows <- labels.survtree(x, pretty = pretty)
    FUN(xy$x, xy$y + 0.5 * cxy, rows[left.child], ...)
  }
  leaves <- if(all) rep(T, nrow(frame)) else frame$var == "<leaf>"
  if(!is.null(label)) {
    if(label == "yval" & !is.null(ylevels))
      stat <- as.character(frame$yval[leaves])
    else if(!is.null(ylevels) && !is.na(lev <- match(label, ylevels)))
      stat <- format(signif(frame$yprob[leaves, lev], 
                            digits = digits))
    else stat <- format(signif(frame[leaves, label], digits = digits))
    FUN(xy$x[leaves], xy$y[leaves] - 0.5 * cxy, stat, ...)
  }
  invisible()
}

"text.survtree.second"<-
function(x, splits = T, label = "leaf.numbers", FUN = text, all = T, pretty
	 = NULL, digits = .Options$digits - 3, ...)
{
  survtree.second.test(x)
  if(label == "leaf.numbers") {
    xy <- treeco(x)
    text(xy, labels = labels.survtree.second(x), adj = 1, srt = 90, ...)
  } else {
    text.tree(x, splits, label, FUN, all, pretty, digits, ...)
  }
}

"vu.analysis.1"<-
c(".C 1", "3.2   Growing a survival tree", "", 
	"The intention of the analysis is summarized in the", 
	"following 'formula':", "", 
	"    LM.form <- formula( t ~ pt + tnm + anz + lap + lrg", 
	"                           + sex + sm + dm + age)", "", 
	"For the function 'survtree' we have to declare", 
	"continuous variables and variables with \"few\"", 
	"levels. This is necessary in order to use special", 
	"adjustment procedures to correct effects of multiple", 
	"testing (Lausen and Schumacher (1992)).", "", 
	"    LM.fac <- c(rep(F,7),T,T)", "", 
	"(7 \"False\"  and 2 \"True\" values according to the", 
	" 9 predictor variables in the formula above)")

"vu.analysis.2"<-
c(".C 1", "", "A 'survtree' object can now be created in two ways:", "", 
	"  - Automatically:", 
	"    LM.tree <- survtree (LM.form, LM, LM$z, LM.fac)", "", 
	"  - Interactively:", 
	"    LM.tree <- survtree (LM.form, LM, LM$z, LM.fac,", 
	"                         ask.flag = 3)", "", 
	"The parameter 'ask.flag' determines the depth of", 
	"interactivity in the classification tree.", "", 
	"The 'survtree' object 'LM.tree' can be visualized", 
	"and labeled by", "", "    plot.survtree(LM.tree)", 
	"    text.survtree(LM.tree, pretty = T)")

"vu.contents.1"<-
c(".C 1", "Contents                                          ", "", 
	"1.   Introduction to CART-SD ", 
	"2.   Implementation in S-Plus : Overview", 
	"3.   Analysis with CART-SD : Example", 
	"3.1   The Data : Liver Metastases", 
	"3.2   Growing a survival Tree", "3.3   Pruning ", 
	"3.4   Kaplan-Meier curves", 
	"3.5   Visualizing the prognostic information", 
	"3.6   \"Staging\" with CART-SD", "", "References")

"vu.contents.2"<-
c(".C 1", "                        Page", "", "                        1 ", 
	"                        2", "                        3", 
	"                        3", "                        5", 
	"                        7", "                        10", 
	"                        12", "                        13", "", 
	"                        16")

"vu.datademo.1"<-
c(".C 1", "3. Analysis with CART-SD : Example", 
	"3.1 The Data : Liver Metastases", "", 
	"The CART-SD functions will be demonstrated by", 
	"using data from patients with liver metastases", 
	"(LM, S-Plus 'data.frame' is also called LM).", 
	"Patients with Liver Metastases from a colorectal", 
	"primary tumor without other distant metastases", 
	"will be studied.", "", 
	"The survival time (t) is measured in months. The", 
	"censoring variable (z) has value 1 if death was", 
	"observed, otherwise value 0. The influence of the", 
	"following covariates on survival will be studied:", "")

"vu.datademo.2"<-
c(".C 1", "        ", "        ", "minimum ", "1.quant.", "median  ", 
	"mean    ", "3.quant.", "maximum ")

"vu.datademo.3"<-
c(".C 1", " Diameter of the", " largest LM (dm)", ":     18  ", 
	":     53  ", ":     60  ", ":     60.1", ":     68  ", 
	":     88  ")

"vu.datademo.4"<-
c(".C 1", "Age of", "patient (age)", "    1", "    2", "    4", "    4.7", 
	"    6", "    20")

"vu.datademo.5"<-
c(".C 1", "(Continuation)", "", "Diagnosis of LM", "(sm)", "", "Sex (sex)", 
	"", "", "Locoregional", "disease (lrg)", "", "Lobar involvement", 
	"(lap)", "", "Number of Liver", "Metastases (anz)", "", "", "", 
	"TNM of primary", "tumor (tnm)", "", "", "Localization of the", 
	"primary tumor (pt)")

"vu.datademo.6"<-
c(".C 1", "", "", "  0 : synchron with primary tumor", 
	"  1 : metachron to primary tumor", "", "  1 : male", 
	"  2 : female", "", "  0 : no LRG", "  1 : LRG", "", 
	"  1 : unilobar", "  2 : bilobar", "", "  0 : singular", 
	"  1 : 2-5 LM", "  2 : 6-20 LM", "  3 : diffuse", "", "  0 : local",
	"  1 : regional", "  2 : distant", "", "  1 : Colon ", 
	"  2 : Rectum")

"vu.finally.1"<-
c(".C 1", "", "The number of reasonable \"stages\" is reduced to 3 or 4.", 
	"According to this, we can create pruned subtrees by:", "", 
	"    LM.stage.3 <- prune.survtree(LM.tr.5.1.sec, k = 15)", 
	"    LM.stage.4 <- prune.survtree(LM.tr.5.1.sec, k = 6)", "", 
	"Finally the staging can be visualized by plotting the", 
	"\"Kaplan-Meier\" curves of every stage and listing the", 
	"number of related nodes of the starting tree LM.tree", 
	"that are combined in the defined stages. Using the", 
	"output of 'graph.survtree', it is possible to identify", 
	"the numbers of the relevant final nodes listed next to", 
	"the \"Kaplan-Meier\" curves with the corresponding", 
	"patient group: 'graph.survtree' shows the node number,", 
	"the number of patients belonging to this node and the", 
	"splitting-process that leads to each final node.")

"vu.grouping.1"<-
c(".C 1", "3.4   Kaplan-Meier curves", "", 
	"These \"pruning\" tools will now be applied to study", 
	"prognostic grouping of patients with liver", 
	"metastases.  A larger sequence of subtrees as shown", 
	"in the last window may be used to answer the question", 
	"of which cost parameter alpha* gives the tree of", 
	"\"right size\".", "", 
	"To this end, one studies the \"Kaplan-Meier\" curves of", 
	"the terminal nodes of each subtree of the sequence", 
	"under consideration.", "", 
	"Now twelve plots of \"Kaplan-Meier\" curves for an", 
	"increasing sequence of the \"split-complexity\"", 
	"parameter alpha will be studied. The range of", 
	"interest for alpha is selected from the 'LM.prune'", "object.", "",
	"")

"vu.grouping.2"<-
c(".C 1", "", "A subjective judgment will be applied to determine", 
	"the point in the sequence where statistical artifacts", 
	"coming from data effects disappear and where the", 
	"survival curves represent prognostic information.     ", 
	" From this point on, choosing higher alpha values", 
	"would coarsen the prognostic information.", "", 
	"The following S-Plus commands produce the plot of", "interest:", 
	"", "    par (mfrow = c(3,4))", "    for ( i in 1:12 )", "    {", 
	"        h.tr <- prune.survtree(LM.tree,", 
	"                    k=LM.prune$k[48+i])", 
	"        h.sv <- surv.fit(LM$t, LM$z,", 
	"                    as.factor(h.tr$where))", 
	"        plot(h.sv, mark.time = F)", "    }")

"vu.implementation.1"<-
c(".C 1", "2. Implementation in S-Plus : Overview", "", 
	"The following new CART-SD functions and objects", 
	"in S-Plus will be presented:", "", 
	"1.) New classes of objects for survival data:", 
	"    'survtree', 'survtree.sequence', 'survtree.second'", "", 
	"2.) A function to generate 'survtree' objects:", 
	"    'survtree(...)'", "", 
	"3.) Simple plotting and labeling procedures:", 
	"    'plot.survtree(...)', 'text.survtree(...)'", "", 
	"4.) An effective pruning function producing an", 
	"    object of class 'survtree.sequence':", 
	"    'prune.survtree(...)'", "")

"vu.implementation.2"<-
c(".C 1", "", "", "5.) A sophisticated plotting function to organize", 
	"    the information of the survival tree visually:", 
	"    'graph.survtree'", "", 
	"6.) different tools useful for working with", 
	"    'survtree'-objects:", 
	"    'predict.survtree', 'snip.survtree', '[.survtree',", 
	"    'browser.survtree', 'edit.survtree', ...", "", 
	"As far as possible we tried to be consistent with", 
	"the approach to tree-based models for continuous", 
	"or categorical responses as described by Clark and", 
	"Pregibon (1991).", 
	"Therefore, a user of CART functions can handle", 
	"most of the new CART-SD functions immediately,", 
	"without additional learning.")

"vu.interest.1"<-
c(".C 1", "3.5   Visualizing the prognostic information", "", 
	"There is one candidate of interest: The tree with", 
	"alpha = 5.104. Using 'graph.survtree' gives the", 
	"chance to organize the relevant prognostic information", 
	"visually. Each node of the tree is presented by a box", 
	"whose width is proportional to the size of the relevant", 
	"patient group.", 
	"'graph.survtree' is an interactive function that", 
	"makes the user choose between a certain survival", 
	"percentile or survival time. Choosing the median", 
	"survival time organizes the produced plot as follows:", 
	"each box representing a node of the tree will be", 
	"centered over its median survival time. Choosing a", 
	"survival time will center the boxes over the relevant", 
	"survival probability.", "", 
	"Choosing one candidate of interest is done by", "", 
	"    LM.tr.5.1 <- prune.survtree(LM.tree, k = 5.1)", 
	"    graph.survtree(LM.tr.5.1, LM$t, LM$z)")

"vu.introduction.1"<-
c(".C 1", "1. Introduction to CART-SD", "", 
	"Tree-based methods often yield rules for", 
	"classification and prediction that are easy to", 
	"interpret for a wide variety of applications.", "", 
	"Interest in tree-based methods for censored data", 
	"usually arises from the need of clinical researchers", 
	"to define interpretable prognostic classification", 
	"rules both for understanding the prognostic structure", 
	"and for designing future clinical trials.", "", 
	"Therefore, clinical researchers are often interested", 
	"in forming a small number of patient groups with", 
	"differing prognoses.", "", "")

"vu.introduction.2"<-
c(".C 1", "", "", "To solve this multivariate problem there is one", 
	"principal route to take: Start with a", 
	"\"stratified Log-Rank-Test\" procedure and end ", 
	"with a Cox regression model.", "", 
	"CART for Survival Data (CART-SD) is an extension", 
	"of the \"stratified Log-Rank-Test\" and gives", 
	"results that are important for assessing ", 
	"the results of the Cox model. Therefore, CART-SD", 
	"fills the gap between the rudimentary explorative", 
	"data analysis of the \"stratified Log-Rank-Test\"", 
	"and the elegant but difficult Cox model.", "", 
	"To make CART-SD a flexible tool, we implemented", "it in S-Plus.")

"vu.ongoing.1"<-
c(".C 1", "3.6   \"Staging\" with CART-SD", "", 
	"The main interest of the ongoing analysis is a", 
	"prognostic grouping of the patients into 3 or 4", 
	"groups of \"bad\", \"good\" and one or two groups of", 
	"\"moderate\" prognoses. These groups are called", 
	"\"stages\". The graph given in the last window gives", 
	"some idea about the patients belonging to the", 
	"different stages. But CART-SD can be applied a", 
	"second time in order to establish the \"stage\"", "definitions:", 
	"", "A \"stage\" will be defined by combining the terminal", 
	"nodes of a tree that define \"bad\", \"moderate\" and", 
	"\"good\" prognoses. Taking the numbers of the terminal", 
	"nodes as a nominal covariate, 'survtree' will be", 
	"applied to the situation of one single covariate.", 
	"This work is done by the function 'survtree.second':", "", 
	"    LM.tr.5.1.sec <- survtree.second (LM.tr.5.1, ", 
	"                                time = LM$t,", 
	"                                zensor = LM$z)", 
	"    LM.tr.5.1.sec.prune <-", 
	"                      prune.survtree(LM.tr.5.1.sec)", "", 
	"These two objects can be plotted as usual:")

"vu.prune"<-
c(".C 1", "", "The function 'prune.survtree' calculates the sequence", 
	"of optimally pruned subtrees:", "", 
	"    LM.prune <- prune.survtree(LM.tree)", "", 
	"The functional dependency between the \"split-", 
	"complexity\" alpha and the number of inner nodes IN", 
	"can be plotted:", "", "    plot.survtree.sequence (LM.prune)", "", 
	"The function 'prune.survtree' can also be used to", 
	"extract the optimally pruned subtree for a given", 
	"complexity parameter alpha*.  The next commands are", 
	"an example of how to plot the optimally pruned", 
	"subtree for different values of alpha*:", "", 
	"    for ( i in 1:6 ) {", "        tr <- prune.survtree(LM.tree,", 
	"                k = LM.prune$k[55 + i])", 
	"        plot.survtree(tr)", 
	"        text.survtree(tr, pretty = T)", "    }")

"vu.references"<-
c(".C 1", "References", "", "Breiman L., Friedman J.H., Olshen R.A., and", 
	"Stone, C.J., (1984).", "Classification and Regression Trees.", 
	"Wadsworth International Group, Belmont CA.", "", 
	"Lausen B. and Schumacher M., (1992).", 
	"Maximally selected rank statistics.", "Biometrics, 48:73-85", "", 
	"Miller R. and Siegmund D. (1982).", 
	"Maximally selected chi-squared statistics", 
	"Biometrics, 38:1011-1016", "", "Clark L. and Pregibon D. (1991).", 
	"Tree--Based Models in J. M. Chambers", 
	"and T. Hastie (editors), Statistical Models in S", 
	"Wadsworth International Group Pacific Grove,", "California")

"vu.strategy.1"<-
c(".C 1", "3.3   Pruning", "", "The CART strategy is based on two ideas:", 
	"", "    1.Splitting : grow an oversized tree", "", 
	"    2.Pruning   : prune the oversized tree to get", 
	"                  a tree expressing the relevant", 
	"                  information contained in the data", "", 
	"The pruning algorithm is based on the idea of a", 
	"\"split-complexity-measure\" G_alpha(T) introduced by ", 
	"LeBlanc and Crowley (1993). Summing up all log-rank", 
	"values of the inner nodes of a tree T gives G(T).", 
	"This value is weighted with a cost factor alpha for", 
	"each element of the set of inner nodes (IN)", "of the tree T:", "",
	"    G_alpha(T) = G(T) - alpha | IN |.")

"vu.strategy.2"<-
c(".C 1", "", "", "If alpha is small, then the cost of a large number", 
	"of splits is small and the tree maximizing G_alpha(T)", 
	"will be large. As alpha increases, the tree maximizing", 
	"G_alpha(T) will have fewer nodes until finally it", 
	"consists only of the root node.", "", 
	"Given alpha* there is only one subtree T* maximizing", 
	"G_alpha*. T* is the optimally pruned subtree of T for", 
	"parameter alpha*. Varying alpha from 0 to infinity", 
	"gives a uniquely determined sequence of optimally", 
	"pruned subtrees.", "", "", "", "", "")

"vu.titlepage"<-
c(".C 1", ".CE 16", ".L 2", "Survival Trees in", "S-Plus", "", "", ".S", 
	"Georg Haupt, Ulrich Mansmann", 
	"Institut fuer medizinische Statistik", 
	"und Informationsverarbeitung", "", "Klinikum Benjamin Franklin", 
	"Freie Universitaet Berlin", "Hindenburgdamm 30", 
	"12200 Berlin, Germany", "", 
	"Tel.: ++49 30/ 8445 2093, FAX: ++49 30/ 834 7751", 
	"e-mail: haupt@ukbf.fu-berlin.de")
