.BG
.FN prune.survtree
.TL
Split-complexity Pruning of `survtree' Object
.DN
Determines a nested sequence of subtrees of the supplied `survtree' by
recursively "snipping" off the least important splits, based upon the
split-complexity measure.
.PP
If `k' is supplied, the optimal subtree is returned.
.CS
prune.survtree (survtree, k, newdata, zensor.newdata, fest.alpha = 0)
.RA
.AG survtree
fitted model object of class `survtree'.
This is assumed to be the result of function `survtree'.
.OA
.AG k
split-complexity parameter defining a specific subtree of `tree'.
.AG newdata
data frame upon which the sequence of split-complexity subtrees is evaluated.
.AG zensor.newdata
vector of status values.  The values have to be `0' or `1', in which
case `0' means censored and `1' means uncensored (dead).
.AG fest.alpha
Undocumented!
.RT
if `k' is supplied, a `tree' object is returned
that minimizes the split-complexity measure for that `k'.
Otherwise, an object of class `survtree.sequence' is returned.
The object contains the following components:
.RC size
number of terminal nodes in each tree in the split-complexity pruning sequence.
.RC deviance
split-complexity of each tree in the split-complexity pruning sequence.
.RC k
the value of the split-complexity pruning parameter of each tree in 
the sequence.
.DT
The response as well as the predictors referred to in the right side
of the formula in `survtree' must be present by name in `newdata'.
These data are dropped down each tree in the split-complexity sequence
and the new split-complexity is calculated.  A `plot()' method exists
for objects of this class.  It displays the value of the
split-complexity for each subtree in the split-complexity sequence.
An additional axis displays the values of the split-complexity
parameter at each subtree.
.SA
`survtree', `edit.survtree', `predict.survtree', `graph.survtree'.
.EX
# determine the split complexity pruning sequence for the tree 'LM.tree'
# for the creation of 'LM.tree' see 'help(survtree)'
LM.prune <- prune.survtree(LM.tree)

# plot this object
plot(LM.prune)

# get a pruned subtree of 'LM.tree' with k = 5.1 (this is the tree with
# "number of inner nodes" = 11. This is so because
# LM.prune$k[56] = 5.096066535 < 5.1 < 5.103522789 = LM.prune$k[57]
# LM.prune$size[56] = 11
LM.tree.5.1 <- prune.survtree(LM.tree, k = 5.1)


# now follows an example on how to work with prune.survtree when a 
# second data set is available. First split 'LM' into two data sets:
LM.1 <- LM[1:400,]
LM.2 <- LM[401:622,]

# create a 'survtree' object on 'LM.1', prune it with k = 4.0,
# and store the 'survtree.sequence' of this pruned survival tree
LM.1.tree <- survtree(LM.form, LM.1, LM.1$z, LM.fac)
LM.1.tree.4.0 <- prune.survtree(LM.1.tree, k = 4.0)
LM.1.tree.4.0.prune <- prune.survtree(LM.1.tree.4.0)
 
# now create a 'survtree.sequence' object on 'LM.2' but with using the 
# 'survtree.sequence' defined by 'LM.1.tree.4.0'
LM.2.prune.by.LM.1.tree.4.0 <- prune.survtree(LM.1.tree.4.0, newdata = LM.2, zensor.newdata = LM.2$z)

# create a survival tree on 'LM.2', but use splitting rules defined by
# LM.1.tree.4.0
LM.2.tree.by.LM.1.tree.4.0 <- predict.survtree(LM.1.tree.4.0, newdata = LM.2, zensor.newdata = LM.2$z, type = "tree")

# create a 'survtree.sequence' object on this tree
# note that object 'LM.2.prune.by.LM.1.tree.4.0' is different from 
# object 'LM.2.prune.by.LM.2.tree'. 
LM.2.prune.by.LM.2.tree <-  prune.survtree(LM.2.tree.by.LM.1.tree.4.0)

.KW survtree
.WR
