#include <S.h>
#ifdef WIN32
#  include <newredef.h>
#endif

#undef DOUBLE
#undef INT
#undef VOID

/******************
	DATEI
	DOU_SPLI.C : zu einer mindestens ordinal skalierten
	             Variablen wird der
	             maximierende Split-Punkt bestimmt;
******************/

#include "import.h"

#include "global.h"
#undef GLOBAL
#define GLOBAL static

#include "tree.h"
#include "dou_spli.h"
#include "set_spli.h"
#include "log_rank.h"
#include "newgrow.h"
#include "loggexe.h"
#include "usertalk.h"

#include "export.h"
#include "maxsel.h"
#include "my_s.h"
#include "newpred.h"
#include "newprune.h"
#include "portab.h"
#include "set_spli.h"

#define FREE(x) free(x)
#define MALLOC(x,y) malloc(x * y)


/******************
	LOKALE FUNKTIONEN 
	dou_compar : Vergleichsfunktion fuer 2 DOUBLE-Werte,
	             wird "qsort" als Parameter uebergeben;
******************/

LOCAL int dou_compar (DOUBLE * x1, DOUBLE * x2);

LOCAL int dou_compar (DOUBLE * x1, DOUBLE * x2)
{
	if( *x1 < *x2 )
		return(-1);
	else
		if( *x1 > *x2 )
			return(1);
		else
			return(0);
} /* dou_compar */

/******************
	GLOBALE FUNKTIONEN 
	dou_split : zu einer mindestens ordinal skalierten
	            Variablen wird der
	            maximierende Split-Punkt bestimmt,
*******************/

GLOBAL DOUBLE dou_split(  /* Rueckgabewert ist der 
                             Wert des maximierten Split-
                             Kriteriums                    */
	UWORD *index_arr,      /* Zeiger auf array der Laenge
	                          "anzahl" mit den globalen 
	                          Indizes der Faelle 
	                          im momentanen Knoten          */
	UWORD anzahl,          /* Anz. der Faelle im 
	                          momentanen Knoten             */
	UWORD var_split,       /* Index der Variable zu der der
	                          Split-Punkt gesucht wird      */
	BOOLEAN * ret_li_true, /* Zeiger auf Rueckgabearray der
	                          Laenge "anzahl" mit den Werten
	                          "TRUE", wenn der entsprechende 
	                          Fall in den linken Knoten 
	                          wandert, sonst "FALSE"        */
	UWORD * ret_li_anzahl, /* Zeiger auf Anz. der Faelle im
	                          linken Knoten                 */
	DOUBLE * ret_val_split /* Zeiger auf den maximierenten 
	                          Split-Kriteriumswert          */
)
{
  DOUBLE
    * x_sortet     = NULL,
    best_val_split = 0.0,
    akt_val_split  = 0.0,
    best_val       = 0.0,
    akt_val        = 0.0,
    last_val_split = 0.0;
  UWORD
    best_li_anzahl = 0,
    akt_li_anzahl  = 0,
    j = 0,
    i = 0;
  BOOLEAN
    * best_li_true = NULL,
    * akt_li_true = NULL;
		
  x_sortet = (DOUBLE *) MALLOC(anzahl, sizeof(DOUBLE));
  for( i = 0; i < anzahl; i++)
    {
      x_sortet[i] = xvars_global[nobs_global * var_split + 
				index_arr[i]];
    }
  qsort(x_sortet, anzahl, sizeof(DOUBLE), dou_compar);

  if(x_sortet[0] != x_sortet[anzahl - 1])
    {
      akt_li_true = (BOOLEAN *) MALLOC(anzahl, sizeof(BOOLEAN));
      best_li_true = (BOOLEAN *) MALLOC(anzahl, sizeof(BOOLEAN));

      last_val_split = x_sortet[0];
      for(i = 1; i < anzahl ; i++)
	{
	  akt_val_split = x_sortet[i];

	  if( (akt_val_split) != last_val_split)
	    {
	      memset(akt_li_true,FALSE,sizeof(BOOLEAN)*anzahl);
	      akt_li_anzahl = 0;
	      akt_val_split = 
		( (akt_val_split) + last_val_split ) / 2.0;
	      last_val_split = x_sortet[i];

	      for(j = 0; j < anzahl; j++)
		{
		  if( xvars_global[nobs_global * var_split + index_arr[j]] 
		      < akt_val_split )
		    {
		      akt_li_anzahl++;
		      akt_li_true[j] = TRUE;
		    }
		}

	      if (
		  (
		   (akt_li_anzahl >= mincut_global) &&
		   ((anzahl - akt_li_anzahl) >= mincut_global)
		   ) &&
		  (
		   !(
		     (fact_fl_global[var_split] != 0) && 
		     (
		      (((1.0 * akt_li_anzahl) / (1.0 * anzahl)) <= left_global) ||
		      (((1.0 * akt_li_anzahl) / (1.0 * anzahl)) >= right_global)
		      )
		     )
		   )
		  )
		{
		  akt_val = log_rank(index_arr, anzahl, akt_li_true, akt_li_anzahl);
		  if( akt_val > best_val )
		    {
		      best_val       = akt_val;
		      best_li_anzahl = akt_li_anzahl;
		      memcpy(best_li_true,akt_li_true,sizeof(BOOLEAN) * anzahl);
		      best_val_split = akt_val_split;
		    }
		}
	    } /* if( (akt_val_split) != last_val_split) */
	} /* for(i = 1; i < anzahl ; i++) */
		
      if(best_val != 0.0)
	{
	  *ret_li_anzahl = best_li_anzahl;
	  memcpy(ret_li_true,best_li_true,sizeof(BOOLEAN) * anzahl);
	  *ret_val_split = best_val_split;
	}
      FREE(akt_li_true);
      FREE(best_li_true);
    } /* if(x_sortet[0] != x_sortet[anzahl - 1]) */
  else
    {
      best_val = 0.0;
    }
  FREE(x_sortet);
  return(best_val);
} /* dou_split */
/******************
	DATEI
	GLOBAL.C : enthaelt einige allgemeine
	           Definitionen, aber vor allem
	           die Funktionen zur Mengenbehandlung;
******************/


/******************
	GLOBALE FUNKTIONEN 
	Mengen-Routinen
******************/

GLOBAL VOID setcpy (SET set1, SET set2)
{
	memmove (set1, set2, SETSIZE * sizeof (ULONG));
} /* setcpy */

GLOBAL VOID setall (SET set)
{
	memset (set, 0xFF, SETSIZE * sizeof (ULONG));
} /* setall */


GLOBAL VOID setclr (SET set)
{
	memset (set, 0x00, SETSIZE * sizeof (ULONG));
} /* setclr */

GLOBAL VOID setnot (SET set)
{
	REG WORD 
		i = 0;

	for (i = 0; i < SETSIZE; i++) 
		set [i] = ~ set [i];
} /* setnot */

GLOBAL VOID setand (SET set1,  SET set2)
{
	REG WORD 
		i = 0;

	for (i = 0; i < SETSIZE; i++) 
		set1 [i] &= set2 [i];
} /* setand */

GLOBAL VOID setor (SET set1,  SET set2)
{
	REG WORD 
		i = 0;

	for(i = 0; i < SETSIZE; i++) 
		set1 [i] |= set2 [i];
} /* setor */

GLOBAL VOID setxor (SET set1,  SET set2)

{
	REG WORD 
		i = 0;

	for(i = 0; i < SETSIZE; i++) 
		set1 [i] ^= set2 [i];
} /* setxor */

GLOBAL VOID setincl (SET  set, WORD elt)
{
	if ((0 <= elt) && (elt <= SETMAX)) 
		set [elt / 32] |= (1L << (elt % 32));
} /* setincl */

GLOBAL VOID setexcl (SET  set, WORD elt)
{
	if ((0 <= elt) && (elt <= SETMAX)) 
		set [elt / 32] &= (~ (1L << (elt % 32)));
} /* setexcl */

GLOBAL BOOLEAN setin (SET  set, WORD elt)
{
	if ((0 <= elt) && (elt <= SETMAX))
		return ((set [elt / 32] & 
			(1L << (elt % 32))) ? TRUE : FALSE);
	else
		return (FALSE);
} /* setin */

GLOBAL BOOLEAN setcmp (SET set1,  SET set2)
{
	REG WORD 
		i = 0;
	REG BOOLEAN 
		res = FALSE;

	for (res = TRUE, i = 0; res && (i < SETSIZE); i++)
		res = (set2 != NULL ? set1 [i] == set2 [i] : set1 [i] == 0);

	return (res);
} /* setcmp */

GLOBAL WORD setcard (SET set)

{
	REG WORD 
		i    = 0,
		j    = 0,
		card = 0;
	REG ULONG l;

	for (i = card = 0; i < SETSIZE; i++)
		for (j = 0, l = set [i]; j < 32; j++)
		{
			if (l & 1) card++;
			l >>= 1;
		} /* for, for */

	return (card);
} /* setcard */

GLOBAL BOOLEAN set2inset1 (SET set1,  SET set2)
{
	REG WORD
		anz = 0;
	SET 
		setres;
	
	setcpy(setres,set1);
	anz = setcard(setres);
	setor(setres,set2);
	if(anz == setcard(setres))
		return(TRUE);
	else
		return(FALSE);
} /* set2inset1 */

GLOBAL VOID setasstring (SET set, STRING setstring)
{
	STRING
		alphabet;
	REG WORD
		i = 0,
		j = 0,
		i_setstring = 0;
	ULONG
		l = 0;
		
	strcpy(alphabet,"abcdefghijklmnopqrstuvwxyz");
	
	for (i = 0; i < SETSIZE; i++)
	{
		l = set[i];
		for (j = 0; j < 32; j++)
		{
			if (l & 1)
			{
				setstring[i_setstring] = alphabet[i*32 + j];
				i_setstring++;
			}
			l >>= 1;
		}
	}
	setstring[i_setstring] = 0;
} /* setasstring */

/******************
	DATEI
	LOG_RANK.C : berechnet den Wert
	             des Log-Rank-Testes fuer zensierte Daten,
							 auserdem wird die Signifikanz des Wertes 
							 mit der Miller-Siegmund Formel
							 bestimmt;
******************/

#define FREE(x) free(x)
#define MALLOC(x,y) malloc(x * y)


LOCAL DOUBLE stdnorm (DOUBLE w);
LOCAL int y_compar (Y_STRUCT * y1, Y_STRUCT * y2);

LOCAL DOUBLE stdnorm (DOUBLE w)
{
	DOUBLE 
		helfer = 0.0;

	helfer = exp(-0.5*w*w);
	helfer *= 1/sqrt(2*PI);
	return(helfer);
} /* stdnorm */

LOCAL int y_compar (Y_STRUCT * y1, Y_STRUCT * y2)
{
	if( y1->time < y2->time )
		return(-1);
	else
		if( y1->time > y2->time )
			return(1);
		else
			if( y1->zensor == y2->zensor )
				return(0);
			else
				if( y1->zensor != TRUE )
					return(1);
				else
					return(-1);
} /* y_compar */

GLOBAL DOUBLE p_chi_normal (DOUBLE x)
{
	DOUBLE
		p = 0.0,
		xx;
	LONG
		d;	

	d = 1L;
	xx = x;
	p_chi_S(&xx,&d,&p);
	return(p);

} /* p_chi_normal */

GLOBAL DOUBLE q_chi_normal (DOUBLE x)
{
	DOUBLE
		q = 0.0,
		xx;
	LONG
		d;	

	d = 1L;
	xx = x;
	q_chi_S(&xx,&d,&q);
	return(q);
} /* q_chi_normal */

GLOBAL DOUBLE chi_mi_si (DOUBLE w, DOUBLE links, DOUBLE rechts)
{
	DOUBLE 
		helfer = 0.0,
		alpha_norm = 0.0,
		alpha = 0.0;

	if(w != 0.0)
	{
		helfer = stdnorm(w);
		alpha = 4 * helfer / w;
		alpha += 
			(
				helfer * (w - 1/w) *
				log(
					(rechts/(1 - rechts)) / 
					(links/(1 - links)) 
				)
			);
	}
	else
	{
		alpha = 1.0;
	}
	return(alpha);
} /* chi_mi_si */

GLOBAL DOUBLE log_rank(
	UWORD *index_arr, 
	UWORD anzahl, 
	BOOLEAN *akt_li_true,
	UWORD akt_li_anzahl)
{
	Y_STRUCT
		last_y_died,
		* y_sortet     = NULL;
	DOUBLE
		u_0            = 0.0,
		i_0            = 0.0,
		res            = 0.0;
	UWORD
		z_li_anz = 0,
		z_re_anz = 0,
		d_i  = 0,
		d_2i = 0,
		n_i  = 0,
		n_2i = 0,
		i    = 0;

	y_sortet = (Y_STRUCT *) MALLOC(anzahl, sizeof(Y_STRUCT));
	memset(y_sortet,0,sizeof(Y_STRUCT) * anzahl);

	for(i = 0; i < anzahl; i++)
	{
		(y_sortet[i]).time   = (y_global[index_arr[i]]).time;
		(y_sortet[i]).zensor = (y_global[index_arr[i]]).zensor;
		(y_sortet[i]).group  = akt_li_true[i];
	}
	
	qsort(y_sortet, anzahl, sizeof(Y_STRUCT), y_compar);

	for( i = 0; i < anzahl; i++)
	{
		if((y_sortet[i]).zensor == 0)
			if((y_sortet[i]).group  == TRUE)
				z_li_anz++;
			else
				z_re_anz++;
	}

	if((z_li_anz != akt_li_anzahl) &&
		(z_re_anz != anzahl - akt_li_anzahl))
	{
		n_2i = akt_li_anzahl;
		n_i = anzahl;

		i = 0;
		while ( i < anzahl )
		{
			last_y_died.zensor = (y_sortet[i]).zensor;
			while( (last_y_died.zensor != TRUE ) &&
				( i < anzahl ) )
			{
				n_i--;
				if( (y_sortet[i]).group == TRUE )
					n_2i--;
				i++;
				last_y_died.zensor = (y_sortet[i]).zensor;
			}

			d_i = 0;
			d_2i = 0;
			last_y_died.time = (y_sortet[i]).time;
			while( ( last_y_died.time == (y_sortet[i]).time ) &&
				( y_sortet[i].zensor == TRUE ) &&
				( i < anzahl ) )
			{
				d_i++;
				if( (y_sortet[i]).group == TRUE )
					d_2i++;
				i++;
			}

			if ( (n_i != 0) && ((n_i - 1) != 0))
			{
				u_0 = u_0 + (d_2i - ((1.0 * d_i * n_2i) / (1.0 * n_i)));
				i_0 = i_0 + 
					(1.0 * d_i * (n_i - d_i) * n_2i * (n_i - n_2i)) /
					(1.0 * n_i * n_i * (n_i - 1) );
			}
			n_2i -= d_2i;
			n_i  -= d_i;			
		} /* while ( i < anzahl ) */
		FREE(y_sortet);

		if(i_0 != 0.0)
			res = u_0 * u_0 / i_0;
		else
			res = -1.0;

	}
	else
	{
		res = -1.0;
	}

	return(res);
} /* log_rank */
/******************
	DATEI
	LOGGEXE.C : allgemeine Loggingroutinen
******************/


LOCAL FILE *fpointer = NULL;

LOCAL VOID loggfinit (VOID);

LOCAL VOID loggfinit (VOID)
{
	time_t timep;

	fpointer = fopen(datei_name_global[0], "w");
	timep = time(NULL);
	fprintf(fpointer,"\n\n\n%s\n", ctime(&timep));
	atexit (loggfexit);
} /* loggfinit */


GLOBAL VOID loggfexit (VOID)
{
	if (fpointer != NULL)
		fclose(fpointer);
	fpointer = NULL;
} /* loggfexit */

GLOBAL VOID loggexe(LONGSTR string, BOOLEAN fileflag)
{
	printf("%s",string);
	if(fileflag == TRUE)
		if (log_flag_global == TRUE)
			loggfexe(string);
} /* loggexe */

GLOBAL VOID loggfexe(LONGSTR string)
{
	if (fpointer == NULL)
		loggfinit();
	fprintf(fpointer,"%s",string);
} /* loggfexe */
/******************
	DATEI
	MAXSEL.C : hier wird das Baumwachstum erledigt,
	           fuer jede Variable wird der maximale 
	           Schnitt-Punkt bestimmt, dann wird 
	           der Baum an dem besten dieser 
	           Schnittpunkten besten geteilt, 
	           der Vorgang rekursiv weitergefuehrt, 
	           bis ein Knoten nicht mehr geteilt 
	           werden kann;
******************/


#define FREE(x) free(x)
#define MALLOC(x,y) malloc(x * y)

LOCAL LONGSTR logstring;

LOCAL WORD get_int (VOID);
LOCAL VOID wait(VOID);

LOCAL VOID wait(VOID)
{
    WORD dummy;

	fflush(stdin);
	fflush(stdout);
	readline_S(&dummy);
	fflush(stdin);
	fflush(stdout);
}

LOCAL WORD get_int (VOID)
{
	WORD
		j;
		
	fflush(stdin);
	fflush(stdout);
	readline_S(&j);
	fflush(stdin);
	fflush(stdout);
	return(j);
}

GLOBAL VOID maxsel (TREE *treaktu)
{
	STRING
		string_dummy;
	WORD
		lines = 0,
		ans;
	UWORD
		*li_anz_arr,
		i = 0,
		j = 0,
		k = 0,
		l = 0,
		vor_lev = 0,
		lefti = 0,
		righti = 0,
		best_li_anzahl = 0,
		akt_li_anzahl  = 0,
		best_var_split = 0,
		akt_var_split  = 0;
	DOUBLE
		*test_arr,
		*alpha_arr,
		*val_arr,
		best_alpha     = 100.0,
		akt_alpha      = 0.0,
		best_test      = 0.0,
		akt_test       = 0.0,
		best_val_split = 0.0,
		akt_val_split  = 0.0;
	BOOLEAN
		* arr_li_true = NULL,
		* best_li_true = NULL,
		* akt_li_true  = NULL;
	SET
		*set_arr,
		best_set_split,
		akt_set_split;
	TREE *treehelfer = NULL;
		
	/*	setbuf(stdin,NULL);
		setlinebuf(stdout); */

	li_anz_arr = (UWORD*) MALLOC(anz_xvars_global, sizeof(UWORD));
	test_arr  = (DOUBLE*) MALLOC(anz_xvars_global, sizeof(DOUBLE));
	alpha_arr = (DOUBLE*) MALLOC(anz_xvars_global, sizeof(DOUBLE));
	val_arr   = (DOUBLE*) MALLOC(anz_xvars_global, sizeof(DOUBLE));
	set_arr    = (SET*)   MALLOC(anz_xvars_global, sizeof(SET));

	memset(li_anz_arr,0,sizeof(UWORD) * anz_xvars_global);
	memset(test_arr,0,sizeof(DOUBLE) * anz_xvars_global);
	memset(alpha_arr,0,sizeof(DOUBLE) * anz_xvars_global);
	memset(val_arr,0,sizeof(DOUBLE) * anz_xvars_global);
	memset(set_arr,0,sizeof(SET) * anz_xvars_global);

	if ( treaktu->anzahl >= minsize_global ) 
	{
		arr_li_true  = (BOOLEAN *) MALLOC(treaktu->anzahl * anz_xvars_global, sizeof(BOOLEAN));
		best_li_true = (BOOLEAN *) MALLOC(treaktu->anzahl, sizeof(BOOLEAN));
		akt_li_true  = (BOOLEAN *) MALLOC(treaktu->anzahl, sizeof(BOOLEAN));

		for( akt_var_split = 0; akt_var_split < anz_xvars_global; 
			akt_var_split++)
		{
			if(xlevels_global[akt_var_split] == 0)
			{
				setclr(set_arr[akt_var_split]);
				setclr(akt_set_split);

				akt_test = dou_split(
					treaktu->index_arr,
					treaktu->anzahl,
					akt_var_split,
					akt_li_true,
					& akt_li_anzahl,
					& akt_val_split);
				if(fact_fl_global[akt_var_split] != 0)
				{
					if(akt_test < 1.3)
					{
						akt_alpha = 1.0;
						akt_test = 0.0;
					}
					else
					{
						akt_alpha = chi_mi_si (sqrt(akt_test),left_global,right_global);
						akt_test = q_chi_normal(1.0 - akt_alpha);
					}
				}
				else
				{
					akt_alpha = 1.0 - p_chi_normal(akt_test);
				}
				test_arr  [akt_var_split] = akt_test;
				alpha_arr [akt_var_split] = akt_alpha;
				val_arr   [akt_var_split] = akt_val_split;
				li_anz_arr[akt_var_split] = akt_li_anzahl;
			}
			else
			{
				val_arr   [akt_var_split] = 0.0;
				akt_val_split = 0.0;
				
				akt_test = set_split(
					treaktu->index_arr,
					treaktu->anzahl,
					akt_var_split,
					akt_li_true,
					& akt_li_anzahl,
					akt_set_split);
				akt_alpha = 1.0 - p_chi_normal(akt_test);

				test_arr  [akt_var_split] = akt_test;
				alpha_arr [akt_var_split] = akt_alpha;
				setcpy(set_arr[akt_var_split],akt_set_split);
				li_anz_arr[akt_var_split] = akt_li_anzahl;
			}
			memcpy(arr_li_true + 
				(treaktu->anzahl * akt_var_split),
				akt_li_true, sizeof(BOOLEAN) * treaktu->anzahl);
			
			if(
				(
					(akt_alpha == 0.0) &&
					(best_alpha == 0.0) &&
					(akt_test > best_test)
				) ||
				(
					(best_alpha != 0.0) &&
					(akt_alpha < best_alpha)
				)
			)
			{
				best_alpha     = akt_alpha;
				memcpy(best_li_true,akt_li_true,
					sizeof(BOOLEAN) * treaktu->anzahl);
				best_li_anzahl = akt_li_anzahl;
				best_val_split = akt_val_split;
				setcpy(best_set_split,akt_set_split);
				best_var_split = akt_var_split;
				best_test      = akt_test;
			}
		} /* for( akt_var_split = 0; akt_var_split < anz_xvars_global ... */

		if(best_test != 0.0)
		{
			ans = 0;
			if(ask_flag_global >= 1)
			{
				if(( ((double) treaktu->id) * 1.0) < pow(2.0,(double) ask_flag_global))
				{
					ans = -2;
					if(treaktu->id == 1)
					{
						info();
						wait();
						printf("\n\n");

					}

					while((ans < 0) || (ans > anz_xvars_global))
					{
						for(j = 0; j < 70; j++)
						{
							sprintf((char *)logstring,"#");
							loggexe(logstring,ans == -2);
						}							
						sprintf((char *)logstring,"\n");
						loggexe(logstring,ans == -2);

						sprintf(string_dummy, STRING_NODE_NR);
						sprintf((char *)logstring,"%s %lu\n",string_dummy, treaktu->id);
						loggexe(logstring,ans == -2);

						for(j = 0; j < 70; j++)
						{
							sprintf((char *)logstring,"#");
							loggexe(logstring,ans == -2);
						}							
						
						sprintf((char *)logstring, STRING_1ST_ROW);
						loggexe(logstring,ans == -2);

						lines = 4;

						sprintf((char *)logstring, STRING_LINE);
						loggexe(logstring,ans == -2);

						lines++;

						for(j = 0; j < anz_xvars_global; j++)
						{
							if(alpha_arr[j] != 1.0)
							{
/*
								if(lines >= 23)
								{
									lines = 0;
									printf(STRING_GO_ON);
									wait();
								}
*/
 								strncpy(string_dummy,xstring_global[j],
									min(strlen(xstring_global[j]) + 1,11));
								string_dummy[min(strlen(xstring_global[j]) + 1,11)] = EOS;

								sprintf((char *)logstring,"|%3u|%s ",j + 1,string_dummy);
								loggexe(logstring,ans == -2);

								if((strlen(xstring_global[j]) + 1) < 11)
									for(k = 0; k < (11 - (strlen(xstring_global[j]) + 1)); k++)
									{
										sprintf((char *)logstring," ");
										loggexe(logstring,ans == -2);
									}

								sprintf((char *)logstring,"|%8u|%9u|", li_anz_arr[j],
									(UWORD) (treaktu->anzahl - li_anz_arr[j]));
								loggexe(logstring,ans == -2);

								if(xlevels_global[j] == 0)
								{
									sprintf((char *)logstring,"%10.5lf",	val_arr[j]);
									loggexe(logstring,ans == -2);

								}
								else
								{
									vor_lev = 0;
									for(l = 0; l < j; l++)
										vor_lev += xlevels_global[l];

									strncpy(string_dummy,xlstring_global[vor_lev],
										min(strlen(xlstring_global[vor_lev]),9));
									string_dummy[min(strlen(xlstring_global[vor_lev]),9)] = EOS;

									sprintf((char *)logstring,"%s",string_dummy);
									loggexe(logstring,ans == -2);

									if(setin(set_arr[j],0) == TRUE)
									{
										sprintf((char *)logstring,"*");
										loggexe(logstring,ans == -2);
									}
									else
									{
										sprintf((char *)logstring," ");
										loggexe(logstring,ans == -2);
									}
									if((strlen(xlstring_global[vor_lev]) + 1) < 9)
										for(k = 0; k <= (9 - (strlen(xlstring_global[vor_lev]) + 1)); k++)
										{
											sprintf((char *)logstring," ");
											loggexe(logstring,ans == -2);
										}
								}
								sprintf((char *)logstring,
									"|%9.5lf|%9.5lf|", test_arr[j], alpha_arr[j]);
								loggexe(logstring,ans == -2);

								if(best_var_split == j)
								{
									sprintf((char *)logstring,"<--\n");
									loggexe(logstring,ans == -2);
								}
								else
								{
									sprintf((char *)logstring,"\n");
									loggexe(logstring,ans == -2);
								}

								lines++;

								if(xlevels_global[j] != 0)
								{
									for(k = 1; k < xlevels_global[j]; k++)
									{
										strncpy(string_dummy,xlstring_global[vor_lev + k],
											min(strlen(xlstring_global[vor_lev + k]),9));
										string_dummy[min(strlen(xlstring_global[vor_lev + k]),9)] = EOS;
										sprintf((char *)logstring,
											"|   |           |        |         |%s",string_dummy);
										loggexe(logstring,ans == -2);


										if(setin(set_arr[j],k) == TRUE)
										{
											sprintf((char *)logstring,"*");
											loggexe(logstring,ans == -2);
										}
										else
										{
											sprintf((char *)logstring," ");
											loggexe(logstring,ans == -2);
										}

										if((strlen(xlstring_global[vor_lev]) + 1) < 9)
											for(l = 0; l <= (9 - (strlen(xlstring_global[vor_lev]) + 1)); l++)
											{
												sprintf((char *)logstring," ");
												loggexe(logstring,ans == -2);
											}
                                        sprintf((char *)logstring, "|         |         |\n",string_dummy);
										loggexe(logstring,ans == -2);

										lines++;
/*
										if(lines >= 23)
										{
											lines = 0;
											printf(STRING_GO_ON);
											wait();
										}
*/
									}
								}

								sprintf((char *)logstring, STRING_LINE);
								loggexe(logstring,ans == -2);

								lines++;
							} /* if(alpha_arr[j] != 1.0) */
						} /* for(j = 0; j < anz_xvars_global; j++) */
						printf(STRING_NR_OF_VAR);

						ans = get_int ();

						if(ans >= 1) 
						{
							if(ans <= anz_xvars_global)
							{
								if(alpha_arr[ans - 1] == 1.0)
								{
									info_no_select_variable();
									wait();
									ans = -2;
								}
							}
							else
							{
								info_no_exist_variable();
								wait();
								ans = -2;
							}
						}
						else
						{
							switch (ans) 
							{
								case -1 : 
									info();
									wait();
									printf("\n\n");
									break;
								case -2 :
									treehelfer = treaktu;
									while(treehelfer->treprev != NULL)
									{
										treehelfer = treehelfer->treprev;
									}
									printf(STRING_TREE);
									treprint(treehelfer);
									treiniprint();
									printf(STRING_GO_ON); fflush(stdout);
									wait();
									printf("\n\n");
									break;
								case 0 :
									break;
								default : 
									info_no_correct_answer();
									wait();
									printf("\n\n");
							}
							if (ans < 0)
								ans = -1;
						}
					} /* while(ans < 0) */

					sprintf(string_dummy, STRING_CHOSEN);
					sprintf((char *)logstring,"%s %u\n\n",string_dummy, ans);
					loggexe(logstring,TRUE);

				} /* if((treaktu->id * 1.0)< pow(2.0,ask_flag_gobal)) */ 
				else
					ans = 0;
			} /* if(ask_flag_global >= 1) */

			if(ans != 0)
			{
				best_alpha     = alpha_arr[ans -1];
				memcpy(best_li_true,
					arr_li_true + (treaktu->anzahl * (ans - 1)),
					sizeof(BOOLEAN) * treaktu->anzahl);
				best_li_anzahl = li_anz_arr[ans - 1];
				best_val_split = val_arr[ans - 1];
				setcpy(best_set_split,set_arr[ans - 1]);
				best_var_split = ans - 1;
				best_test      = test_arr[ans - 1];
			}
			
			treadd(treaktu, TRUE);
			treadd(treaktu, FALSE);
	
			(treaktu->treleft)->index_arr = 
				(UWORD *) MALLOC (best_li_anzahl, sizeof(UWORD));
			(treaktu->treright)->index_arr = 
				(UWORD *) MALLOC ((treaktu->anzahl - best_li_anzahl), sizeof(UWORD));
		
			lefti = righti = 0;
			for ( i = 0; i < treaktu->anzahl; i++)
			{
				if ( best_li_true[i] == TRUE )
				{
					((treaktu->treleft)->index_arr)[lefti] = 
						(treaktu->index_arr)[i];
					lefti++;
				}
				else
				{
					((treaktu->treright)->index_arr) [righti] = 
						(treaktu->index_arr) [i];
					righti++;
				}
			}
			FREE(arr_li_true);
			FREE(best_li_true);
			FREE(akt_li_true);

			treaktu->alpha     = best_alpha;
			treaktu->krit_val  = best_test;
			treaktu->val_split = best_val_split;
			setcpy(treaktu->set_split,best_set_split);
			treaktu->var_split = best_var_split;

			(treaktu->treleft)->anzahl = lefti;
			(treaktu->treright)->anzahl = righti;

			maxsel(treaktu->treleft);
			maxsel(treaktu->treright);
		}
		else
		{
			FREE(arr_li_true);
			FREE(best_li_true);
			FREE(akt_li_true);
		}
	}
	FREE(li_anz_arr);
	FREE(test_arr);
	FREE(alpha_arr);
	FREE(val_arr);
	FREE(set_arr);

} /* maxsel */
/******************
	DATEI
	MY_S.C : 
******************/


#ifdef __unix__
#ifdef INT
#undef INT 
#endif
#ifdef DOUBLE
#undef DOUBLE
#endif
#ifdef VOID
#undef VOID
#endif

#include <S.h>

#ifdef INT
#undef INT 
#define INT int
#endif
#ifdef DOUBLE
#undef DOUBLE
#define DOUBLE double
#endif
#ifdef VOID
#undef VOID
#define VOID void
#endif
#endif 


LOCAL BYTE *pmodes[] = {"double","integer"};
LOCAL LONG plengths [] = {1,1};
LOCAL BYTE *pnames[] = { "q","df"} ;

LOCAL BYTE *qmodes[] = {"double","integer"};
LOCAL LONG qlengths [] = {1,1};
LOCAL BYTE *qnames[] = { "p","df"} ;

GLOBAL VOID chi_call(VOID **Spchisq_p, DOUBLE *x, DOUBLE *p)
{
	LONG
		d;
	d = 1L;
	Spchisq = *Spchisq_p;
	p_chi_S(x,&d,p);
}
	
GLOBAL VOID p_chi_S(DOUBLE *x, LONG *d, DOUBLE *p)
{
	VOID
		*in[2];
	VOID
		*out[1];

	in[0] = (VOID *)x;
	in[1] = (VOID *)d;
	call_S(Spchisq,2L,in,pmodes,plengths,pnames,1L,out);
	*p = ((DOUBLE *) out[0])[0];
}

GLOBAL VOID q_chi_S(DOUBLE *x, LONG *d, DOUBLE *q)
{
	VOID
		*in[2];
	VOID
		*out[1];

	in[0] = (VOID *)x;
	in[1] = (VOID *)d;
	call_S(Sqchisq,2L,in,qmodes,qlengths,qnames,1L,out);
	*q = ((DOUBLE *) out[0])[0];
}

GLOBAL VOID readline_S(WORD * number)
{
	VOID
		*out[1];

	call_S(Sreadline,0L,NULL,NULL,NULL,NULL,1L,out);
	*number = ((WORD *) out[0])[0];
}

/******************
	DATEI
	NEWGROW.C : Schnittstelle zu S-Plus,
	            Baumwachstum;
******************/

#define FREE(x) free(x)
#define MALLOC(x,y) malloc(x * y)

VOID newgrow(
	/* hier die ersten Eingabevariblen                */

	DOUBLE * X,      /* Covariablenmatrix der Groesse
	                    "nobs" * "ncloX"              */
	DOUBLE * Y,      /* Zielvariablen-array           */        
	DOUBLE * w,      /* array mit Gewichten
	                    fuer Beobachtungen, wird noch
	                    nicht benutzt. In spaeterer 
	                    Version vorgesehen            */
	WORD * XYlevels, /* array der Groesse "ncolX" + 1
	                    X oder Y nominal ? Anz. der
	                    Merkmalsauspraegungen         */
	BYTE **Xstring,  /* array mit den Namen der 
	                    X-Variablen                   */
	BYTE **Xlstring,  /* array mit den Namen der Levels
	                    X-Variablen                   */
	WORD * Yzensor,  /* Zensierungs-array             */
	WORD * nobs,     /* Anz. der Beobachtungseinheiten*/
	WORD * ncolX,    /* Anz. der X-Variablen          */
	WORD * Ask_flag, /* Flag, ob ein Split durch den
	                    Benutzer interaktiv
	                    beeinflusst werden soll */
	WORD * Log_flag, /* Flag, ob diese Beeinflussung in
	                    einer Datei gespeichert werden
	                    soll                          */
	BYTE **Datei_name,  /* Name dieser Datei          */

	/* hier die ersten Variablen mit Rueckgabewert    */

	WORD * node,     /* Knotennummern-array           */
	WORD * var,      /* array mit den Nummern
	                    der Variablen die den Knoten
	                    verzweigen lassen             */
	BYTE ** cutleft, /* array mit dem
	                    Intervall von "var",
	                    das nach links geht           */
	BYTE ** cutright,/* array mit dem
	                    Intervall von "var",
	                    das nach rechts geht          */
	WORD * n,        /* array mit den
	                    Anz. der Faelle der Knoten    */
	DOUBLE * dev,     /* Pseudo-Deviance, wird durch 
	                     Summierung der log-rank Werte
	                     erhalten                     */
	DOUBLE * log_rank,/* "log_rank"-values der Knoten */
	DOUBLE * log_alph,/* "log_rank"-Siginifikanz 
	                     von "log_rank"               */

	/* hier nochmal einige Eingabevariablen           */

	WORD * minsize,  /* minimale Anzahl mit der ein 
	                    Knoten noch gespalten 
	                    werden darf                   */
	WORD * mincut,   /* minimale Anzahl die jeder der 2
	                    Nachfolgerknoten noch 
	                    haben muss                    */
	DOUBLE * left_t, /* linke Grenze aus der Miller-
	                    Siegmund-Formel zur Berechnung
	                    der Signifikanz des maximal se-
	                    lektierten Split-Punktes      */
	DOUBLE * right_t,/* rechte Grenze                 */

	VOID **Spchisq_p,/* Wahrscheinichkeits-
	                    chi-Quadrat-Funktion von S_Plus*/

	VOID **Sqchisq_p,/* Quantil-
	                    chi-Quadrat-Funktion von S_Plus*/

	VOID **Sreadline_p,/*  Eingabe-funktion */
	
	WORD * fact_flag,/* Array mit boolschen Werten, die
	                    anzeigen, ob die X-Variablen
	                    geordnete Faktoren oder 
	                    stetige Variablen sind        */
	/* hier nochmal einige mit Rueckgabewert           */

	WORD * nnode,    /* Anzahl der Knoten insgesamt   */
	WORD * where     /* array mit Indizes, zeigt fuer
	                    jede Beobachtungseinheit auf
	                    seinen Endknoten              */
)
{
	UWORD
		i = 0;
	TREE 
		*trefirst = NULL;	
		
	minsize_global = *minsize;
	mincut_global  = *mincut;
	left_global    = *left_t;
	right_global   = *right_t;
	anz_xvars_global = *ncolX;
	nobs_global    = *nobs;
	ask_flag_global = *Ask_flag;
	log_flag_global = *Log_flag;
	
	xlevels_global = (UWORD* ) XYlevels;
	xstring_global = (BYTE**) Xstring;
	xlstring_global = (BYTE**) Xlstring;
	datei_name_global = (BYTE**) Datei_name;
	xvars_global   = (DOUBLE *) X;
	fact_fl_global = (UWORD* ) fact_flag;
	
	Spchisq = *Spchisq_p;
	Sqchisq = *Sqchisq_p;
	Sreadline = *Sreadline_p;

	y_global = (Y_STRUCT *) MALLOC (nobs_global, sizeof(Y_STRUCT));
	memset(y_global,0,sizeof(Y_STRUCT) * nobs_global);
	
	for ( i = 0; i < nobs_global ; i++)
	{
		y_global[i].time   = Y[i];
		y_global[i].zensor = Yzensor[i];
		y_global[i].group  = FALSE;
	}

	trefirst = tremal();
	treinit(trefirst);
	trefirst->anzahl = nobs_global;
	(trefirst->index_arr) = (UWORD *) MALLOC (nobs_global, sizeof(UWORD));

	for( i = 0; i < nobs_global; i++)
	{
		(trefirst->index_arr)[i] = i;
	}

	
	maxsel(trefirst);
	
/*	treprint(trefirst);
*/	
	*nnode = 0;
	trefill(trefirst, nnode, node, var, n,
		dev, log_rank, log_alph, cutleft, cutright, where);
	(*nnode)++;

	trereset(trefirst);
	loggfexit();
	FREE(y_global);
} /* newgrow */

/******************
	DATEI
	NEWPRED.C : 
******************/


VOID newpred(
	/* hier die Eingabevariblen                       */

	WORD * nnode,    /* Anzahl der Knoten insgesamt   */
	WORD * node,      /* Knotennummern-array           */
	DOUBLE * Y,       /* Array mit den Test-Sample-Y  */
	DOUBLE * Yzensor, /* Array mit der Zensur-Variable
	                     zu den Test-Sample-Y         */
	DOUBLE * Where,   /* Array mit Indizes zu jedem 
	                     Test-Sample-Y, die Indexe zeigen
	                     auf einen Knoten von "node"
	                     (siehe einige Zeilen vorher) */
	WORD * Y_anz,     /* Anzahl der Faelle des Test-Sample */

	VOID **Spchisq_p ,/* Wahrscheinichkeits-
	                     chi-Quadrat-Funktion von S_Plus*/

	/* hier die Variablen mit Rueckgabewert           */

	DOUBLE * new_log_rank, /* "log_rank"-values der Knoten */
	DOUBLE * new_log_alph, /* "log_rank"-Siginifikanz 
	                          von "log_rank"               */
	DOUBLE * new_dev       /* Pseudo-deviance , enthaelt fuer 
														jeden Knoten die Summe der log-rank-Werte
														seiner Nachfolger und seines eigenen */
)
{
	UWORD 
		*index_arr,
		li_anzahl,
		anzahl;
	BOOLEAN 
		* li_true;
	WORD
		i           = 0,
		j           = 0,
		k           = 0;
	DOUBLE 
		val = 0.0;		

	if (Spchisq_p != NULL)
	{
		Spchisq = *Spchisq_p;
	}

	nobs_global = *Y_anz;

	y_global = (Y_STRUCT *) MALLOC(nobs_global, sizeof(Y_STRUCT));
	memset(y_global,0,sizeof(Y_STRUCT) * nobs_global);

	for ( i = 0; i < nobs_global ; i++)
	{
		y_global[i].time   = Y[i];
		y_global[i].zensor = Yzensor[i];
		y_global[i].group  = FALSE;
	}

	index_arr = (UWORD *)   MALLOC((*Y_anz), sizeof(UWORD));
	li_true   = (BOOLEAN *) MALLOC((*Y_anz), sizeof(BOOLEAN));

	for( i = 0; i < (*nnode); i++)
	{
		if ( 
			((i + 1) < (*nnode)) && 
			((node[i] * 2) == node[i+1])
		)
		{
			memset(index_arr,0,sizeof(UWORD) * (*Y_anz));
			anzahl = 0;
			li_anzahl = 0;

			j = i + 1;

			while( 
				(						
					(node[j] > node[i]) && 
					(j < (*nnode)) &&
					(node[j] != (node[i] + 1))
				) ||
				(
					(j < (*nnode)) &&
					(i == 0)
				)
			)
			{
				for(k = 0; k < (*Y_anz); k++)
				{
					if(Where[k] == (j + 1))
					{
						index_arr[anzahl] = k;
						anzahl++;
					}
				}
				j++;
			}

			memset(li_true,FALSE,sizeof(BOOLEAN) * anzahl);

			j = i + 1;
			while( node[j] != ((2 * node[i]) + 1))
			{
				for(k = 0; k < anzahl; k++)
				{
					if(Where[index_arr[k]] == (j + 1))
					{
						li_true[k] = TRUE;
						li_anzahl++;
					}
				}
				j++;
			}
			new_log_rank[i] = log_rank(
				index_arr, anzahl, li_true, li_anzahl);
			if(new_log_rank[i] == -1.0)
				new_log_rank[i] = 0.0;
		}
		else
		{
			new_log_rank[i] = 0.0;
		}
		if(Spchisq_p != NULL)
			new_log_alph[i] = 1.0 - p_chi_normal(new_log_rank[i]);
			
	} /* for( i = 0; i < (*nnode); i++) */

	FREE(li_true);
	FREE(index_arr);
	FREE(y_global);

	for( i = 0; i < (*nnode); i++)
	{
		j = i + 1;
		new_dev[i] = new_log_rank[i];

		if(j < (*nnode))
		{
			while( 
				(						
					(node[j] > node[i]) && 
					(j < (*nnode)) &&
					(node[j] != (node[i] + 1))
				) ||
				(
					(j < (*nnode)) &&
					(i == 0)
				)
			)
			{
				new_dev[i] += new_log_rank[j];
				j++;
			}
		}
	}
		
} /* newpred */
/******************
	DATEI
	NEWPRUNE.C : Schnittstelle zu S-Plus,
	             Baumbeschneidung;
******************/

VOID newprune(
	/* hier die Eingabevariblen                       */

	WORD * nnode,    /* Anzahl der Knoten insgesamt   */
	WORD * node,      /* Knotennummern-array           */
	DOUBLE * val_log_rank,   /* Pseudo-Deviance, wird durch 
	                     Summierung der log-rank Werte
	                     erhalten                     */
	DOUBLE * Y,       /* Array mit den Test-Sample-Y  */
	DOUBLE * Yzensor, /* Array mit der Zensur-Variable
	                     zu den Test-Sample-Y         */
	DOUBLE * Where,   /* Array mit Indizes zu jedem 
	                     Test-Sample-Y, die Indexe zeigen
	                     auf einen Knoten von "node"
	                     (siehe einige Zeilen vorher) */
	WORD * Y_anz,     /* Anzahl der Faelle des Test-Sample */

	DOUBLE * fest_alpha,/* das fuer die geprunten Baeume 
	                     zu verwendende feste alpha   */

	/* hier die Variablen mit Rueckgabewert           */

	DOUBLE * outdev,  /* die "Deviance der jeweiligen
	                     geprunten Baeume             */
	DOUBLE * newdev,  /* die "Deviance" fuer das Test-
	                     Sample                       */
	DOUBLE * alpha,   /* das fuer den geprunten Baum 
	                     charakteristische alpha      */
	WORD * size,      /* die Anzahl der Endknoten im
	                     geprunten Baum               */
	WORD * n,         /* die Anzahl der geprunten 
	                     Baeume insgesamt             */
	WORD * inode      /* Array mit den Indizes der 
	                     beschnittenen Knoten         */
)
{
	WORD
		best_knot = 0,
		akt_knot  = 0,
		best_node   = 0,
		i           = 0,
		j           = 0,
		k           = 0,
		* work_node = NULL;
	DOUBLE
		lastdev = 0.0,
		lastnewdev = 0.0,
		* work_log_rank = NULL,
		* new_log_rank = NULL,
		* new_log_alph = NULL,
		best_log_rank = 0.0,
		akt_log_rank  = 0.0,
		best_g = 1000.0,
		akt_g  = 0.0;
		
	work_node     = (WORD *)   MALLOC ((*nnode), sizeof(WORD));
	work_log_rank = (DOUBLE *) MALLOC ((*nnode), sizeof(DOUBLE));
	new_log_rank  = (DOUBLE *) MALLOC ((*nnode), sizeof(DOUBLE));
	new_log_alph  = (DOUBLE *) MALLOC ((*nnode), sizeof(DOUBLE));
	
	*n = 0;
	size[0]  = *nnode / 2 + 1;
	alpha[0] = 0.0;
	outdev[0] = 0.0;
	
	for( i = 0; i < (*nnode); i++)
		outdev[0] += val_log_rank[i];
	lastdev = outdev[0];

	if(*fest_alpha >= 0.0)
		outdev[0] -= (*fest_alpha * (size[0] - 1.0));

	if(*Y_anz != 0)
	{
		newpred(nnode, node, Y, Yzensor, Where, Y_anz, NULL, new_log_rank, new_log_alph, newdev);
		/*
		for( i = 0; i < (*nnode); i++)
			newdev[0] += new_log_rank[i];
		*/
		if(*fest_alpha >= 0.0)
			newdev[0] -= (*fest_alpha * (size[0] - 1.0));

	} /* if(*Y_anz != 0) */

	for( i = 0; i < (*nnode); i++)
	{
		work_node[i] = 1;
		work_log_rank[i] = val_log_rank[i];
	}
	
	do
	{
		best_g = 1000.0;
		for( i = 0; i < (*nnode); i++)
		{
			if( (work_node[i] == 1) &&
				(work_log_rank[i] != 0.0) )
			{

				akt_knot = 1;
				akt_log_rank = work_log_rank[i];
				j = i + 1;

				if( (work_node[j] == 1) &&
					((node[i] * 2) == node[j]) )
				{

					while( 
						(						
							(node[j] > node[i]) && 
							(j < (*nnode)) &&
							(node[j] != (node[i] + 1))
						) ||
						(
							(j < (*nnode)) &&
							(i == 0)
						)
					)
					{
						if( (work_node[j] == 1) &&
							(work_log_rank[j] != 0.0) )
						{
							akt_knot++;
							akt_log_rank += work_log_rank[j];
						}
						j++;
					}
				} /* if( (work_node[j] == 1) &&	((node[i] * 2) == node[j]) ) */

				akt_g = akt_log_rank * 1.0 / (1.0 * akt_knot);


				if( akt_g < best_g )
				{
					best_node = i;
					best_g = akt_g;
					best_knot = akt_knot;
					best_log_rank = akt_log_rank;
				}
			} /* if(work_node[i] == 1) */
		} /* for( i = 0; i < (*nnode); i++) */

		(*n)++;
		size[*n]  = size[(*n) - 1] - best_knot;
		alpha[*n] = best_g;

		lastdev -= best_log_rank;
		if(*fest_alpha < 0.0)
			outdev[*n] = lastdev - (alpha[*n] * (size[*n] - 1.0));
		else
			outdev[*n] = lastdev - (*fest_alpha * (size[*n] - 1.0));
			
		inode[(*n) - 1] = best_node + 1;

		work_log_rank[best_node] = 0.0;
		if(*Y_anz != 0)
			new_log_rank[best_node] = 0.0;
		j = best_node + 1;
		while( 
			(						
				(node[j] > node[best_node]) && 
				(j < (*nnode)) &&
				(node[j] != (node[best_node] + 1))
			) ||
			(
				(j < (*nnode)) &&
				(best_node == 0)
			)
		)
		{
			work_node[j] = 0;
			work_log_rank[j] = 0.0;
			if(*Y_anz != 0)
				new_log_rank[j] = 0.0;
			j++;
		}
		
		if(*Y_anz != 0)
		{
			newdev[*n] = 0;
			for( i = 0; i < (*nnode); i++)
			{
				if(work_node[i] == 1)
					newdev[*n] += new_log_rank[i];
			}
			if(*fest_alpha < 0.0)
				newdev[*n] -= (alpha[*n] * (size[*n] - 1.0));
			else
				newdev[*n] -= (*fest_alpha * (size[*n] - 1.0));
		}

	} while ( size[*n] > 1 );
	(*n)++;
	FREE(work_node);
	FREE(new_log_rank);
	FREE(new_log_alph);
	FREE(work_log_rank);

} /* newprune */
/******************
	DATEI
	SET_SPLI.C : zu einer nominal skalierten
	             Variablen wird der
	             maximierende Split-Punkt bestimmt;
******************/


/******************
	GLOBALE FUNKTIONEN 
	set_split : zu einer nominal skalierten
	            Variablen wird der
	            maximierende Split-Punkt bestimmt,
*******************/

GLOBAL DOUBLE set_split(  /* Rueckgabewert ist der 
                             Wert des maximierten Split-
                             Kriteriums                    */
	UWORD *index_arr,      /* Zeiger auf array der Laenge
	                          "anzahl" mit den globalen 
	                          Indizes der Faelle 
	                          im momentanen Knoten          */
	UWORD anzahl,          /* Anz. der Faelle im 
	                          momentanen Knoten             */
	UWORD var_split,       /* Index der Variable zu der der
	                          Split-Punkt gesucht wird      */
	BOOLEAN * ret_li_true, /* Zeiger auf Rueckgabearray der
	                          Laenge "anzahl" mit den Werten
	                          "TRUE", wenn der entsprechende 
	                          Fall in den linken Knoten 
	                          wandert, sonst "FALSE"        */
	UWORD * ret_li_anzahl, /* Zeiger auf Anz. der Faelle im
	                          linken Knoten                 */
	SET ret_set_split      /* Menge der Auspraegungen von 
	                          "var_split" im linken Knoten  */
)
{
	DOUBLE
		best_val       = 0.0,
		akt_val        = 0.0;
	UWORD
		werte_anz      = 0,
		best_li_anzahl = 0,
		akt_li_anzahl  = 0,
		j = 0,
		i = 0,
		k = 1;
	BOOLEAN
		* best_li_true = NULL,
		* akt_li_true = NULL;
	SET
		werte_set,
		best_set_split,
		akt_set_split;
		
	setclr(werte_set);
	setclr(best_set_split);
	setclr(akt_set_split);

	for( i = 0; i < xlevels_global[var_split]; i++)
	{
		j = 0;
		while ( ( setin(werte_set,i) == FALSE ) &&
			j < anzahl )
		{
			if( (i + 1) == xvars_global[nobs_global * var_split + index_arr[j]] )
			{
				setincl(werte_set,i);
			}
			j++;
		}
	}
	
	if( (werte_anz = setcard(werte_set)) >= 2)
	{
		akt_li_true = (BOOLEAN *) MALLOC (anzahl, sizeof(BOOLEAN));
		best_li_true = (BOOLEAN *) MALLOC (anzahl, sizeof(BOOLEAN));
		
		if(pow(2.0,werte_anz - 1.0) > 1000.0)
		{
			info_set_spli_aktuell (xstring_global[var_split], pow(2.0,werte_anz - 1.0) );
		}
		for(akt_set_split[0] = 1; 
			akt_set_split[0] < ((werte_set[0] * 1.0) / 2.0);
			(akt_set_split[0])++)
		{
			if((k % 1000) == 0)
			{
				printf(STRING_NUMBER_IS);
				printf("%u\n",k);
				fflush(stdout);
			}
			if(set2inset1(werte_set,akt_set_split) == TRUE)
			{
				k++;
				memset(akt_li_true,FALSE,sizeof(BOOLEAN)*anzahl);
				akt_li_anzahl = 0;

				for(j = 0; j < anzahl; j++)
					{
					if( setin(akt_set_split,
						(WORD) (xvars_global[nobs_global * var_split + index_arr[j]]) - 1))
					{
						akt_li_anzahl++;
						akt_li_true[j] = TRUE;
					}
				}
				if ((akt_li_anzahl >= mincut_global) &&
					((anzahl - akt_li_anzahl) >= mincut_global) )
				{
					akt_val = log_rank(index_arr, anzahl, akt_li_true, akt_li_anzahl);
					if( akt_val > best_val )
					{
						best_val       = akt_val;
						best_li_anzahl = akt_li_anzahl;
						memcpy(best_li_true,akt_li_true,sizeof(BOOLEAN) * anzahl);
						setcpy(best_set_split,akt_set_split);
					}
				}
			}
		} /* for(akt_set_split[0] = 1; */
		if(best_val != 0.0)
		{
			*ret_li_anzahl = best_li_anzahl;
			memcpy(ret_li_true,best_li_true,sizeof(BOOLEAN) * anzahl);
			setcpy(ret_set_split,best_set_split);
		}
		FREE(akt_li_true);
		FREE(best_li_true);
	} /* if( (werte_anz = set_card(werte_set)) >= 2) */
	else
	{
		best_val = 0.0;
	}

	return(best_val);
} /* set_split */
/* Datei : TREE.C :	Standardgeruest zum Umgang mit doppelt 
							verketteten Listen
#include <string.h>
*/

LOCAL WORD lines = 0;

GLOBAL TREE *tremal(VOID)
{
	return ((TREE *) MALLOC (1, sizeof (TREE)));
}
	

GLOBAL VOID treinit(TREE *trefirst)
{
	trefirst->treleft  = NULL;
	trefirst->treright = NULL;
	trefirst->treprev  = NULL;

	trefirst->index_arr      = NULL;

	trefirst->id       = 1;
	trefirst->anzahl   = 0;

	trefirst->alpha    = 0.0;
	trefirst->krit_val = 0.0;
	trefirst->val_split= 0.0;
	setclr(trefirst->set_split);
	trefirst->var_split= 0;
}


GLOBAL VOID treadd(TREE *treaktu,BOOLEAN left)
{
	TREE *trenew;
	
	trenew = tremal();

	trenew->treprev  = treaktu;
	trenew->treleft  = NULL;
	trenew->treright = NULL;

	trenew->index_arr      = NULL;
	
	if (left)
	{
		trenew->id = treaktu->id * 2;
		treaktu->treleft = trenew;
	}
	else /*** jetzt fuer den rechten Ast ***/
	{
		trenew->id = (treaktu->id * 2) + 1;
		treaktu->treright = trenew;
	}
	trenew->alpha     = 0.0;
	trenew->krit_val  = 0.0;
	trenew->val_split = 0.0;
	setclr(trenew->set_split);
	trenew->var_split = 0;
}

GLOBAL VOID trereset(TREE *trefirst)
{
	TREE *helfer;
	
	if ((helfer = trefirst->treleft) != NULL)
	{
		trereset(helfer);
	}
	if ((helfer = trefirst->treright) != NULL)
	{
		trereset(helfer);
	}
	
	FREE(trefirst->index_arr);
	FREE(trefirst);
}

GLOBAL VOID treiniprint(void)
{
	lines = 0;
}

GLOBAL VOID treprint(TREE *trefirst)
{
	STRING
		splitprint,
		splitstring;
	TREE 
		*helfer;
	DOUBLE 
		i = 1.0;
	WORD 
		l = 0,
		k = 0,
		vor_lev = 0;

	while ( (((trefirst->id) * 1.0) / pow(2.0, (i * 1.0)) ) >= 1.0 )
	{
		i += 1.0;
		printf(" ");
	}

	if (trefirst->val_split == 0.0)
	{
		if(trefirst->set_split[0] != 0)
		{
			vor_lev = 0;
			for(l = 0; l < trefirst->var_split; l++)
				vor_lev += xlevels_global[l];

			splitstring[0] = EOS;
			for(k = 0; k < xlevels_global[trefirst->var_split]; k++)
			{
				if(setin(trefirst->set_split,k) == TRUE)
				{
					strcat(splitstring,xlstring_global[vor_lev + k]);
					strcat(splitstring,",");
				}
			}
			splitstring[strlen(splitstring) - 1] = EOS;

			sprintf(splitprint,
				"%s: %s",
				xstring_global[trefirst->var_split],
				splitstring);
		}
		else
		{
			sprintf(splitprint,"<leaf>");
		}
	}
	else
	{
		sprintf(splitprint,
			"%s < %lf",
			xstring_global[trefirst->var_split],
			trefirst->val_split);
	}

	printf("%lu %u %s\n",trefirst->id,trefirst->anzahl,splitprint);

/*
	for ( j = 0; j < trefirst->anzahl; j++ )
	{
		printf(" %d",trefirst->arr[j * 8]);
		if ( (j + 1) % 35 == 0 )
			printf("\n");
	}
	printf("\n");
*/

	if ((helfer = trefirst->treleft) != NULL)
	{
		treprint(helfer);
	}
	if ((helfer = trefirst->treright) != NULL)
	{
		treprint(helfer);
	}
}

GLOBAL VOID trefill(TREE *treaktu,WORD *nnode, WORD *node, 
	WORD *var, WORD *n,
	DOUBLE *dev,
	DOUBLE *log_rank, DOUBLE *log_alph, 
	BYTE **cutleft, BYTE **cutright, WORD *where)
{
	WORD
		akt_nnode   = 0,
		left_nnode  = 0,
		right_nnode = 0;
	UWORD
		i = 0,
		j = 0;
	TREE
		*helfer;
	STRING
		set_string;
	SET
		werte_set;
	BYTE
		*cutstring;
	
	
	node[*nnode] = (WORD) treaktu->id;
	n[*nnode]    = (WORD) treaktu->anzahl;

	if ((helfer = treaktu->treleft) != NULL)
	{
		dev[*nnode]  = treaktu->krit_val;
		var[*nnode]  = (WORD) treaktu->var_split + 1;
		log_rank[*nnode] = treaktu->krit_val;
		log_alph[*nnode] = treaktu->alpha;

		if(xlevels_global[treaktu->var_split] == 0)
		{
			cutstring = (BYTE *) S_alloc(40, sizeof(BYTE));
			sprintf(cutstring,"<%-g",treaktu->val_split);
			cutleft[*nnode]  = cutstring;

			cutstring = (BYTE *) S_alloc(40, sizeof(BYTE));
			sprintf(cutstring,">%-g",treaktu->val_split);
			cutright[*nnode] = cutstring;
		}
		else
		{
			setasstring(treaktu->set_split,set_string);
			cutstring = (BYTE *) S_alloc(40, sizeof(BYTE));
			sprintf(cutstring,":%s",set_string);
			cutleft[*nnode]  = cutstring;

			setclr(werte_set);
			for( i = 0; i < xlevels_global[treaktu->var_split]; i++)
			{
				j = 0;
				while ( ( setin(werte_set,i) == FALSE ) &&
					j < (treaktu->anzahl) )
				{
					if( (i + 1) == xvars_global[nobs_global * 
						(treaktu->var_split) + (treaktu->index_arr)[j]] )
					{
						setincl(werte_set,i);
					}
					j++;
				}
			}
			setxor(werte_set,treaktu->set_split);
			setasstring(werte_set,set_string);
			cutstring = (BYTE *) S_alloc(40, sizeof(BYTE));
			sprintf(cutstring,":%s",set_string);
			cutright[*nnode] = cutstring;
		} /* if-else (xlevels_global[treaktu->var_split] == 0) */

		akt_nnode = *nnode;
		(*nnode)++;
		left_nnode = *nnode;

		trefill(helfer, nnode, node, var, n,
			dev, log_rank, log_alph, cutleft, cutright, where);
		
		dev[akt_nnode] += dev[left_nnode];
		
		helfer = treaktu->treright;
		(*nnode)++;
		right_nnode = *nnode;

		trefill(helfer, nnode, node, var, n,
			dev, log_rank, log_alph, cutleft, cutright, where);

		dev[akt_nnode] += dev[right_nnode];
	}
	else
	{
		dev[*nnode]      = 0.0;
		var[*nnode]      = 0;
		log_rank[*nnode] = 0.0;
		log_alph[*nnode] = 0.0;
		for( i = 0; i < treaktu->anzahl; i++)
		{
			where[((treaktu->index_arr)[i])] = (UWORD) treaktu->id;
		}
	}
}
/******************
	DATEI
	USERTALK.C : Datei fuer die simple user-Schnittstelle
******************/

GLOBAL VOID info_no_select_variable(VOID)
{
	printf("\n---------------------------------------\n");
	printf("- This variable cannot be selected !!!-\n");
	printf("---------------------------------------\n\n");
	printf(STRING_GO_ON); fflush(stdout);
}

GLOBAL VOID info_no_exist_variable(VOID)
{
	printf("\n-----------------------------------\n");
	printf("- This variable does not exist !!!-\n");
	printf("-----------------------------------\n\n");
	printf(STRING_GO_ON); fflush(stdout);
}

GLOBAL VOID info_no_correct_answer(VOID)
{
	printf("\n-----------------------------------\n");
	printf("- No correct answer, try again !!!-\n");
	printf("-----------------------------------\n\n");
	printf(STRING_GO_ON); fflush(stdout);
}
GLOBAL VOID info_set_spli_aktuell (BYTE *xstring_var_split, DOUBLE anz )
{
	printf("The current inspected variable \"%s\"\n",xstring_var_split);
	printf("is a variable of type factor. There are \n");
	printf("%u possible split-sets. \n",(UWORD) anz);
	printf("To show that the program is running and does not break down,\n");
	printf("every 1000th reached split-set is announced on the screen! \n");
	fflush(stdout);
}

GLOBAL VOID info(VOID)
{
    printf("\n\n----------------- Information -----------------------------------\n");
    printf("The variables with the corresponding split-point \n");
    printf("( or split-set if the split-variable is of class factor )\n");
    printf("will be displayed.\n");
    printf("Now you are asked for the number of the variable which \n");
    printf("will be taken as the split-variable in the actual node.\n\n");
    printf("If the split-variable is of type continuous \n");
    printf("( see argument fact.flag in help(survtree) ), \n");
    printf("the log-rank-value is adjusted according to the article \n");
    printf("of Lausen and Schumacher, see help(survtree).\n\n");
    printf("For variables of class factor the elements \n");
    printf("which would go to the left are signed with a \"*\".\n\n");
    printf("The variable with the best log-rank-value is signed \n");
    printf("with \"<--\". \n");
    printf("You can choose this variable by typing the number 0.\n\n");
    printf("If you type \"?\", this Information will be displayed again.\n");
    printf("If you type \"!\", the whole now existing tree will be displayed.\n\n");
    printf("Any wrong input leeds to an error message and the \n");
    printf("variable-listing will be displayed again.\n");
    printf("\n\n----------------- Information -----------------------------------\n\n");
    printf(STRING_GO_ON);  fflush(stdout);
}

